/* 
License:
You are free to reuse any part of the program as long as you
 - Keep this notice with it.
 - If you use it in any of your research that will be made public,
   you also release the derived work from this file to make your
   result reproducible by audience.
 - Written by Bernard Lidicky


This is a 'simple' flag algebra program for generating SDPs.

 
Use:
 Write Makefile in a sibdirecotry that looks like:

CFLAGS= -DG_COLORED_EDGES=2 
SRC=../flag.cpp  
 
 -include ../Makefile.include

And compile. Different targets are available. 

 
Files and file formats:
 
 Problem:
    min/max c_1xG_1 + c_2xG_2 + c_3xG_3
 subject to
    c_4xG_4 + c_5xG_5 + c_6 \geq 0
    c_7xG_7 + c_8xG_8 + c_9 \geq 0
 
 Objective file: (F_edges2__objective.txt)
   c_1 G_1
   c_2 G_2
   c_3 G_3
 0
   c_6
   c_4 G_5
   c_5 G_5
 0
   c_9
   c_7 G_7
   c_8 G_8
 0
 Optional program description (that does not start with a number)

See the the beginning for different defines for compilation.
 
Features: 
  * Many parts of the program use OpenMP. You will experience a significant speedup when running
    the program on a multicore, shared memory machine
  * The whole thing is in one file because we hope that compiler might do some interprocedural optimization
*/

#include <iostream>
#include <fstream>
#include <sstream>
#include <istream>
#include <iterator>
#include <vector>
#include <vector>
#include <utility>
#include <assert.h>
#include <cstring>
#include <algorithm>
#include <cstdarg>
#include <cstdlib>
#include <cmath>
#include <limits> 
#include <unistd.h>
#include <iomanip>  

using namespace std;


/*

// Main settings
 
#define G_COLORED_VERTICES        //=x     number of possible colors for vertices. Using colors (1..x)
#define G_COLORED_VERTICES_BLIND  //       vertex color-blind version     
#define G_COLORED_EDGES           //=x     2-edges have x colors (1...x)
#define G_COLORED_EDGES_BLIND     //       partial edge color blind. Reads permutations of colors from a file
#define G_ORIENTED_EDGES          //       1 is not flipped but all other colors have version with +,- for orientations. i.e, 1 can be used in place of non-edge
#define G_ORIENTED_EDGES_BLIND    //       the actuall orientation does not matter anc may be flipped
#define G_3EDGES                  //       enable 3-edges
#define G_COLORED_3EDGES          //=x     3-edges have colors (1..x)
#define G_COLORED_3EDGES_BLIND    //       permutations of colorings for 3-edges is loaded from a file
#define G_COLORED_3EDGES_PRINT_HUMAN // human friendly output for 3-edges (costs more hardddrive space)
#define G_4EDGES                  //       enable 4-edges
 
#define G_ORDERED_VERTICES        // not working - usefull for permutations
 
#define G_USE_BABER_EQUATIONS     //       enables baber's equations.
 
#define G_TOURNAMENT             // define use as a tournament -  in that case color 1 is skipped when generating graphs. Like forbodding color 1 but more effective

#define G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY // If the program is working with blow-up, it is more effective to generate the small graphs and then do
                                               // blow-ups. The color used for blowing-up will be color 1. Faster than using forbidden subgraphs.
#define USE_COUNT_INSTEAD_OF_DENSITY_IN_OBJECTIVE // When computing the density for objective function, it uses count instead of density. 
                                 // Usefull if you are shooting for exact result and want the SDP to have integers in the input. Alternatively, same could be
                                 // achieved by properly changing the objective function but it gives chance for rounding errors
#define G_USE_PERMITTED_SUBFLAGS // If the number of forbidden subflags of some size is too big, you may enumerate permitted subflags of some size in addition
                                  // to forbidden subflags
#define WRITE_INTEGER_DENSITIES_FOR_LINEAR_CONSTRAINTS  // it will modify the linear constraints by multiplying them by some constant and hope the numbers
                                  // from linear constraints that appear in the sdp will be integers

 
// Tweaks - maybe not used

#define G_REMOVE_DUPLICATES_WHILE_LOADING_FLAGS     // Usefull when loading some 'dirty' input
#define G_CHECK_FOR_FORBIDDEN_WHILE_LOADING_FLAGS   //
#define G_EARLY_FORBIDDEN_TEST_IN_GENERATING  // when coloring edges, test for forbidden graphs after every subgraph 0...x is entirely colored
#define G_FLAG_PRODUCTS_SLOW_LOW_MEMORY       // really slow, dont do this
#define G_USE_LEXMIN_FOR_ISOMORPHISM     // there was an attempt to do this but it does not work fast


#define G_DISABLE_MAIN
 */


#ifndef V
#define V      11  // Maximum number of vertices of flags. If you know you need smaller, use smaller - will be faster and less memory consuming
#endif

#define G_NOT_ALL_FLAGS_USED // Some extra warnings if there are unexpected flags. This should be enabled

#define G_USING_ZERO_AS_ANY_COLOR // If something has color 0, it is considered to have anycolor or being uncolored. Usefull when trying to find extensions of some graph

#define G_BE_BRAVE_AND_IGNORE_SAFETY_ASSERTS // Will be slower, but if not defined. But if you don't trust the program, comment it :-)

#ifndef G_ORDERED_VERTICES
#define USE_REDUCED_TYPES // 4 4 1:123  and 4 4 1:234 are considered redundant and only one is used. This you want to have on. Not having it is stupid unless you have ordered vertices
#endif

#define MAX_FLAG_TYPES  300000 // Just a constant - raise if neded - means different theta


#define DISABLE_UNLABELED_PRODUCTS  // TODO!!



#ifdef G_ORIENTED_EDGES
#ifndef G_COLORED_EDGES
#error Oriented edges only work if normal edges are set up
#endif
#endif


// Copy to +1 counterparts.... to make life easier
#ifdef G_COLORED_VERTICES
#define COLORS_VERTICES (G_COLORED_VERTICES+1)
#endif
#ifdef G_COLORED_EDGES
#define COLORS_EDGES (G_COLORED_EDGES+1)
#endif
#ifdef G_COLORED_3EDGES
#define COLORS_3EDGES (G_COLORED_3EDGES+1)
#endif


#ifndef G_BLOW_UP_COLOR_EDGES
#define G_BLOW_UP_COLOR_EDGES  1
#endif

#ifndef G_BLOW_UP_COLOR_3EDGES
#define G_BLOW_UP_COLOR_3EDGES 1
#endif

#ifndef G_BLOW_UP_COLOR_4EDGES
#define G_BLOW_UP_COLOR_4EDGES 1
#endif


#ifdef G_COLORED_EDGES_BLIND
vector<vector<int> > g_allowed_edgecolor_permutations;  // permutations of colors of edges
#endif

#ifdef G_COLORED_3EDGES_BLIND
vector<vector<int> > g_allowed_3edgecolor_permutations;  // permutations of colors of 3-edges
#endif


#ifdef G_COLORED_VERTICES_BLIND
vector<vector<int> > g_allowed_vertexcolor_permutations;  // permutations of colors of 3-edges
#endif


// This is the main class that holds a flag. 
class flag
{
public:
    flag()
    {
		m_Theta = 0;
		m_vertices = 0;
#ifdef G_ORDERED_VERTICES
        m_labeled_vertices_cnt = 0;
        for(int v = 0; v < V; v++)
        {
            m_labeled[v] = false;
        }
        update_labeledvector();
#endif
    }
    
    virtual ~flag()
    {
        
    }
    
    
    
#ifdef G_ORDERED_VERTICES
    void update_labeledvector()
    {
        int i = 0;
        for (int v = 0; v < m_vertices; v++)
        {
            if (m_labeled[v]) m_labeled_vector[i++] = v;
        }
        assert(i == m_labeled_vertices_cnt);
    }
    
    void set_labeled(const int v)
    {
        assert(v < m_vertices);
        if (is_labeled(v)) return;
        m_labeled[v] = true;
        m_Theta = m_Theta | (1 << v);
        m_labeled_vertices_cnt++;
        update_labeledvector();
    }
    
    void clear_labeled(const int v)
    {
        assert(v < m_vertices);
        if (!is_labeled(v)) return;
        m_labeled[v] = false;
        m_Theta = m_Theta & (~(1 << v));
        m_labeled_vertices_cnt--;
        update_labeledvector();
    }
#endif    
    
    void make_all_vertices_labeled()
    {
#ifdef G_ORDERED_VERTICES
        set_Theta(~(0xFFFFFFFF << m_vertices));
#else
        m_Theta = m_vertices;
#endif
    }
    
    bool is_labeled(const int v) const
    {
        assert(v < m_vertices);
#ifdef G_ORDERED_VERTICES
        return m_labeled[v];
#else
        return  v < m_Theta;
#endif
    }
    
    // returns the number of labeled vertices
    int labeled_vertices_cnt() const
    {
#ifdef G_ORDERED_VERTICES
        return m_labeled_vertices_cnt;
#else
        return m_Theta;
#endif
    }    

    
    // First m_Theta vertices are considered labeled in that order
    void set_Theta(int Theta)
    {
        // if !G_ORDERED_VERTICES, then the labeled vertices are first. Not true for ordered.
#ifndef G_ORDERED_VERTICES
        assert(Theta <= m_vertices);
        m_Theta = Theta;
#else
        m_Theta = 0;
        m_labeled_vertices_cnt = 0;
        for (int v = 0; v < m_vertices; v++)
            if (Theta & (1<<v))
            {
                m_Theta = m_Theta | (1 << v);
                m_labeled[v] = true;
                m_labeled_vertices_cnt++;
            }
            else m_labeled[v] = false;
        update_labeledvector();
#endif
    }
    
	
    int get_Theta()
    {
        return m_Theta;
    }
    
    void set_vertices_and_Theta(int vertices, int Theta)
    {
        set_vertices(vertices);
        set_Theta(Theta);
    }
	
	
    // set the nymber of vertices and clears the graph
    void set_vertices(int vertices)
    {
        assert(vertices <= V);
        m_vertices = vertices;
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM                
        m_minlex_signature = "";
#endif
        
#ifdef G_COLORED_VERTICES
		for (int u = 0; u < V; u++)
		{
			m_color_vertex[u] = 0;
		}
		m_colored_vertices[0] = m_vertices;
		for (int c = 1; c < COLORS_VERTICES; c++) m_colored_vertices[c] = 0;
#endif

		
#ifdef G_COLORED_EDGES
        for (int u = 0; u < V; u++)
        {
	        for (int v = 0; v < V; v++)
	        {
	            m_color_edge[u][v] = 0;
	        }
	    }
        
        m_colored_edges[0] = e();
        for (int c = 1; c < COLORS_EDGES; c++)  m_colored_edges[c] = 0;
#endif

#ifdef G_3EDGES
        for (int u1 = 0; u1 < V; u1++)
            for (int u2 = 0; u2 < V; u2++)
                for (int v1 = 0; v1 < V; v1++)
                    m_3edge[u1][u2][v1] = 0;
        m_3edges_cnt = 0;
#endif
        
#ifdef G_COLORED_3EDGES    
        for (int u1 = 0; u1 < V; u1++)
            for (int u2 = 0; u2 < V; u2++)
                for (int v1 = 0; v1 < V; v1++)
                    m_colored_3edge[u1][u2][v1] = 0;
        m_colored_3edges[0] = e3();
        for (int c = 1; c < COLORS_3EDGES; c++)  m_colored_3edges[c] = 0;
#endif       
        
#ifdef G_4EDGES
        for (int u1 = 0; u1 < V; u1++)
            for (int u2 = 0; u2 < V; u2++)
                for (int v1 = 0; v1 < V; v1++)
                    for (int v2 = 0; v2 < V; v2++)
                        m_4edge[u1][u2][v1][v2] = 0;
        m_4edges_cnt = 0;
#endif
    }



#ifdef G_COLORED_VERTICES
    void color_vertex(int u, int color)
    {
        assert(u < m_vertices);
        assert(color >= 0);
        assert(color < COLORS_VERTICES);
        
        m_colored_vertices[m_color_vertex[u]]--;
        
        m_color_vertex[u] = color;
        
        m_colored_vertices[color]++;
        
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM 
        m_minlex_signature.clear();
#endif      
    }
#endif
    

#ifdef G_COLORED_EDGES
    void color_edge(int u, int v, int color)
    {
#ifdef G_ORIENTED_EDGES
        if (color < 0)
        {
            color_edge(v, u, -color);
            return;
        }        
#endif
    	assert(u != v);
    	assert(u < m_vertices);
    	assert(color >= 0);
        assert(color < COLORS_EDGES);
        
    	int old_color = m_color_edge[u][v];
#ifdef G_ORIENTED_EDGES
        if (old_color < 0) old_color = -old_color;        
#endif        
        
    	m_colored_edges[old_color]--;
    	
#ifdef G_ORIENTED_EDGES
        if (color != 1)   // 1 is not oriented....
        {
            m_color_edge[u][v] =  color;
            m_color_edge[v][u] = -color;
        }
        else
        {
            m_color_edge[u][v] = m_color_edge[v][u] = color;
        }
#else
        m_color_edge[u][v] = m_color_edge[v][u] = color;
#endif
    	
    	m_colored_edges[color]++;

#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM 
        m_minlex_signature.clear();
#endif
    }
#endif
    
    
#ifdef G_3EDGES
    void set_3edge(int x, int y, int z, int state)
    {
        assert (x < m_vertices);
        assert (y < m_vertices);
        assert (z < m_vertices);
        
        if (m_3edge[x][y][z] == state) return;
        m_3edge[x][y][z] = state;
        m_3edge[x][z][y] = state;
        m_3edge[y][x][z] = state;
        m_3edge[y][z][x] = state;
        m_3edge[z][x][y] = state;
        m_3edge[z][y][x] = state;
        if (state == 1) m_3edges_cnt++;
        else m_3edges_cnt--;
        
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM 
        m_minlex_signature.clear();
#endif      
    }
#endif 
    
#ifdef G_COLORED_3EDGES
    void color_3edge(int x, int y, int z, int color)
    {
        
        assert (x < m_vertices);
        assert (y < m_vertices);
        assert (z < m_vertices);
        assert(color >= 0);
        assert(color < COLORS_3EDGES);
        assert (x != y);
        assert (x != z);
        assert (z != y);
        
        if (m_colored_3edge[x][y][z] == color) return;
        m_colored_3edges[m_colored_3edge[x][y][z]]--;
        
        m_colored_3edge[x][y][z] = color;
        m_colored_3edge[x][z][y] = color;
        m_colored_3edge[y][x][z] = color;
        m_colored_3edge[y][z][x] = color;
        m_colored_3edge[z][x][y] = color;
        m_colored_3edge[z][y][x] = color;
        m_colored_3edges[color]++;
        
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM 
        m_minlex_signature.clear();
#endif     
    }
#endif        
 
#ifdef G_4EDGES
    void set_4edge(int x, int y, int z, int w, int state)
    {
        assert (x < m_vertices);
        assert (y < m_vertices);
        assert (z < m_vertices);
        assert (w < m_vertices);
        
        if (m_4edge[x][y][z][w] == state) return;        
        m_4edge[x][y][z][w] = state;
        m_4edge[x][z][y][w] = state;
        m_4edge[y][x][z][w] = state;
        m_4edge[y][z][x][w] = state;
        m_4edge[z][x][y][w] = state;
        m_4edge[z][y][x][w] = state;
        m_4edge[x][y][w][z] = state;
        m_4edge[x][z][w][y] = state;
        m_4edge[y][x][w][z] = state;
        m_4edge[y][z][w][x] = state;
        m_4edge[z][x][w][y] = state;
        m_4edge[z][y][w][x] = state;
        m_4edge[x][w][y][z] = state;
        m_4edge[x][w][z][y] = state;
        m_4edge[y][w][x][z] = state;
        m_4edge[y][w][z][x] = state;
        m_4edge[z][w][x][y] = state;
        m_4edge[z][w][y][x] = state;          
        m_4edge[w][x][y][z] = state;
        m_4edge[w][x][z][y] = state;
        m_4edge[w][y][x][z] = state;
        m_4edge[w][y][z][x] = state;
        m_4edge[w][z][x][y] = state;
        m_4edge[w][z][y][x] = state; 
        if (state == 1) m_4edges_cnt++;
        else m_4edges_cnt--;
        
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM 
        m_minlex_signature.clear();
#endif     
    }
#endif  
    
	
    
    
    
#ifdef G_COLORED_VERTICES
//    void permute_vertex_colors(int color_permutation[])
    void permute_vertex_colors(const vector<int> & color_permutation)
    {
        for (int v = 0; v < m_vertices; v++)
        {
            color_vertex(v, color_permutation[m_color_vertex[v]]);
        }
    }
#endif
    
#ifdef G_COLORED_EDGES_BLIND
    // Makes a permutation of edges of this flag
    // Maybe be rewritten for higher effectivity
    void permute_edge_colors(const vector<int> &color_permutation)
    {
        assert (color_permutation.size() >= COLORS_EDGES);
        
        for (int u = 0; u < m_vertices-1; u++)
            for (int v = u+1; v < m_vertices; v++)
        {
#ifdef G_ORIENTED_EDGES
            if (m_color_edge[u][v] < 0)
            {
                color_edge(u,v,-color_permutation[-m_color_edge[u][v]]);                      
            } else {
                color_edge(u,v,color_permutation[m_color_edge[u][v]]);      
            }
#else
            color_edge(u,v,color_permutation[m_color_edge[u][v]]);      
#endif
        }
    }
#endif 
    
    
#ifdef G_COLORED_3EDGES_BLIND
    // Makes a permutation of 3-edges of this flag
    // Maybe be rewritten for higher effectivity
    void permute_3edge_colors(const vector<int> &color_permutation)
    {
        assert (color_permutation.size() >= COLORS_3EDGES);
        
        for (int u = 0; u < m_vertices-2; u++)
            for (int v = u+1; v < m_vertices-1; v++)
                for (int w = v+1; w < m_vertices; w++)
            {
                color_3edge(u,v,w,color_permutation[m_colored_3edge[u][v][w]]);      
            }
    }
#endif


#ifdef G_ORIENTED_EDGES_BLIND
    void swap_orientation()
    {
        for (int u = 0; u < m_vertices; u++)
        {
            for (int v = 0; v < m_vertices; v++)
            {
                // just reversing orientation
                if (m_color_edge[u][v] != 1) m_color_edge[u][v] = -m_color_edge[u][v];
            }
        }     
    }
#endif

    
// Sequence of functions for checking if two flags have the same type //
private:
    // TODO: This sequence could be rewritten by using early check and no need
    // for creating copies of F - could be more effective.
    bool have_same_type_colorblind_colored_3edges(const flag &h) const
    {
#ifdef G_COLORED_3EDGES_BLIND
        flag F;
        for (int p = 0; p < (int)g_allowed_3edgecolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_3edge_colors(g_allowed_3edgecolor_permutations[p]);
            if (have_same_type_colorblind_colored_edges(F)) return true;            
        }
        return false;
#else
        return have_same_type_colorblind_colored_edges(h);
#endif
    }
    
    
    bool have_same_type_colorblind_colored_edges(const flag &h) const
    {
#ifdef G_COLORED_EDGES_BLIND
        flag F;
        for (int p = 0; p < (int)g_allowed_edgecolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_edge_colors(g_allowed_edgecolor_permutations[p]);
            if (have_same_type_colorblind_oriented_edges(F)) return true;            
        }
        return false;
#else
        return have_same_type_colorblind_oriented_edges(h);
#endif
    }
    
    bool have_same_type_colorblind_oriented_edges(const flag &h) const
    {
#ifdef G_ORIENTED_EDGES_BLIND
        if (have_same_type_colorblind_vertices(h)) return true;
        flag swapped;
        swapped = h;
        swapped.swap_orientation();
        return have_same_type_colorblind_vertices(swapped);
#else
        return have_same_type_colorblind_vertices(h);
#endif
    }
    
    bool have_same_type_colorblind_vertices(const flag &h) const
    {
#ifdef G_COLORED_VERTICES_BLIND
        /* // Old universal version
        int color_permutation[] = {0,1,2,3,4,5,7,8,9,10}; // longer than needed
        
        flag F;
        do {
            F = h; // copy flag
            F.permute_vertex_colors(color_permutation);
            if (have_same_type_not_colorblind(F)) return true;
        } while ( std::next_permutation(color_permutation+1,color_permutation+COLORS_VERTICES));
        return false;
          */
        flag F;
        for (int p = 0; p < (int)g_allowed_vertexcolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_vertex_colors(g_allowed_vertexcolor_permutations[p]);
            if (have_same_type_not_colorblind(F)) return true;            
        }
        return false;        
#else
        return have_same_type_not_colorblind(h);
#endif
    }

    bool have_same_type_not_colorblind(const flag &f) const
    {
        //if (m_Theta != f.m_Theta) return false;
        //if (m_Theta == 0) return true;
        
#ifdef G_COLORED_VERTICES
        // same vertex color
        for (int u = 0; u < m_Theta; u++)
        {
#ifdef G_USING_ZERO_AS_ANY_COLOR
            if (m_color_vertex[u] == 0 || f.m_color_vertex[u] == 0) continue;
#endif            
            if (m_color_vertex[u] != f.m_color_vertex[u]) return false;
        }
#endif
        
        
#ifdef G_COLORED_EDGES
        assert(m_Theta < V);
        for (int u = 0; u < m_Theta; u ++)
            for (int v = u+1; v < m_Theta; v++)
            {
#ifdef G_USING_ZERO_AS_ANY_COLOR
                if (m_color_edge[u][v] == 0 || f.m_color_edge[u][v] == 0) continue;
#endif            
                if (m_color_edge[u][v] != f.m_color_edge[u][v]) return false;
            }
#endif
        
#ifdef G_3EDGES
        for (int x = 0; x < m_Theta-2; x++)
            for (int y = x+1; y < m_Theta-1; y++)
                for (int z = y+1; z < m_Theta; z++)
                {
                    if (m_3edge[x][y][z] != f.m_3edge[x][y][z]) return false;
                }
#endif          
        
#ifdef G_COLORED_3EDGES
        for (int x = 0; x < m_Theta-2; x++)
            for (int y = x+1; y < m_Theta-1; y++)
                for (int z = y+1; z < m_Theta; z++)
                {
#ifdef G_USING_ZERO_AS_ANY_COLOR
                    if (m_colored_3edge[x][y][z] == 0 || f.m_colored_3edge[x][y][z] == 0) continue;
#endif              
                    if (m_colored_3edge[x][y][z] != f.m_colored_3edge[x][y][z]) return false;
                }
#endif            
        
#ifdef G_4EDGES
        for (int x = 0; x < m_Theta-3; x++)
            for (int y = x+1; y < m_Theta-2; y++)
                for (int z = y+1; z < m_Theta-1; z++)
                    for (int w = z+1; w < m_Theta; w++)
                    {
                        if (m_4edge[x][y][z][w] != f.m_4edge[x][y][z][w]) return false;
                    }
#endif
        return true;
    }
    

public:    
    // This is the function to call in general program
    bool have_same_type(const flag &f) const
    {
        if (m_Theta == 0 && f.m_Theta == 0) return true;
        if (labeled_vertices_cnt() != f.labeled_vertices_cnt()) return false;
        
#ifdef G_ORDERED_VERTICES
        flag type_this;
        flag type_f;
        get_type_subflag(type_this);
        f.get_type_subflag(type_f);
        return type_this.is_isomorphic_to(type_f);
#else
        return have_same_type_colorblind_colored_3edges(f);
#endif
    }
    
    
    
    
    
    
    
    
    
    bool is_isomorphic_to_colorblind_colored_3edges(const flag &h) const
    {
#ifdef G_COLORED_3EDGES_BLIND
        flag F;
        for (int p = 0; p < (int)g_allowed_3edgecolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_3edge_colors(g_allowed_3edgecolor_permutations[p]);
            if (is_isomorphic_to_colorblind_colored_edges(F)) return true;            
        }
        return false;  
#else
        return is_isomorphic_to_colorblind_colored_edges(h);
#endif
    }
    
    bool is_isomorphic_to_colorblind_colored_edges(const flag &h) const
    {
#ifdef G_COLORED_EDGES_BLIND
        flag F;
        for (int p = 0; p < (int)g_allowed_edgecolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_edge_colors(g_allowed_edgecolor_permutations[p]);
            if (is_isomorphic_to_colorblind_oriented_edges(F)) return true;            
        }
        return false;  
#else
        return is_isomorphic_to_colorblind_oriented_edges(h);
#endif
    }
    
    bool is_isomorphic_to_colorblind_oriented_edges(const flag &h) const
    {
#ifdef G_ORIENTED_EDGES_BLIND
        if (is_isomorphic_to_colorblind_vertices(h)) return true;
        flag swapped;
        swapped = h;
        swapped.swap_orientation();
        return is_isomorphic_to_colorblind_vertices(swapped);
#else
        return is_isomorphic_to_colorblind_vertices(h);
#endif
    }
    
    bool is_isomorphic_to_colorblind_vertices(const flag &h) const
    {
#ifdef G_COLORED_VERTICES_BLIND
        /*
        int color_permutation[] = {0,1,2,3,4,5,7,8,9,10,11}; // longer than needed
        
        flag F;
        do {
            F = h; // copy flag
            F.permute_vertex_colors(color_permutation);
            if (is_isomorphic_to_not_colorblind(F)) return true;
        } while ( std::next_permutation(color_permutation+1,color_permutation+COLORS_VERTICES));
        return false;
        */
        
        flag F;
        for (int p = 0; p < (int)g_allowed_vertexcolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_vertex_colors(g_allowed_vertexcolor_permutations[p]);
            if (is_isomorphic_to_not_colorblind(F)) return true;            
        }
        return false;   
        
#else
        return is_isomorphic_to_not_colorblind(h);
#endif
    }
    
    bool is_isomorphic_to(const flag &h) const
    {
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM
        // if we have signatires, we are can just use them and that is it
        if (!m_minlex_signature.empty() && !h.m_minlex_signature.empty())
        {
            if (m_minlex_signature.compare(h.m_minlex_signature) != 0) return false;
            return true;            
        }
#endif
        // Just a quick kill
        if (m_vertices != h.m_vertices) 
        {
            return false;
        }
        if (labeled_vertices_cnt() != h.labeled_vertices_cnt())
        {
            return false;
        }
        
        //cerr << "Testing " << h.print() << " and " << print() << endl;
        
        return is_isomorphic_to_colorblind_colored_3edges(h);
    }
	
	
	// perm is mapping vertices of h to vertices of this. pv is a candidate for
	// mapping of v, all vertices before v are already mapped.
	bool is_map_up_to_v_correct(int v, int pv, const int *perm, const flag &h) const
	{
			
#ifdef G_COLORED_VERTICES
        // 0 as a joker color
        if ((m_color_vertex[pv] != h.m_color_vertex[v]) && (m_color_vertex[pv] != 0) && (0 != h.m_color_vertex[v])) return false;
#endif
						
#ifdef G_COLORED_EDGES
      // check edges, 0 as joker
      for (int u = 0; u < v; u++)
         if (m_color_edge[perm[u]][pv] != h.m_color_edge[u][v] && m_color_edge[perm[u]][pv] != 0 && h.m_color_edge[u][v] != 0) return false;        
#endif

#ifdef G_3EDGES
      for (int x = 0; x < v; x++)
         for (int y = x+1; y < v; y++)
         {
             if (m_3edge[perm[x]][perm[y]][pv] != h.m_3edge[x][y][v]) return false;
         }
#endif   

#ifdef G_COLORED_3EDGES
        for (int x = 0; x < v; x++)
            for (int y = x+1; y < v; y++)
            {
                 if (m_colored_3edge[perm[x]][perm[y]][pv] != h.m_colored_3edge[x][y][v])
                 {
                     return false;
                 }
            }
#endif          
        
#ifdef G_4EDGES
        for (int x = 0; x < v; x++)
            for (int y = x+1; y < v; y++)
                for (int z = y+1; z < v; z++)
                {
                    if (m_4edge[perm[x]][perm[y]][perm[z]][pv] != h.m_4edge[x][y][z][v]) return false;
                }
#endif           
        
		return true;
	}

    bool is_mapping_an_equality(int *perm, const flag &h) const
    {
#ifdef G_COLORED_EDGES
        //	check coloring
        for (int u = 0; u < m_vertices; u++)
        {
            for (int x = u+1; x < m_vertices; x++)
            {
                if (m_color_edge[perm[u]][perm[x]] == 0 || h.m_color_edge[u][x]==0) continue;
                if (m_color_edge[perm[u]][perm[x]] != h.m_color_edge[u][x]) return false;
            }
        }
#endif
        
#ifdef G_3EDGES
        for (int x = 0; x < m_vertices; x++)
            for (int y = x+1; y < m_vertices; y++)
                for (int z = y+1; z < m_vertices; z++)
                {
                    if (m_3edge[perm[x]][perm[y]][perm[z]] != h.m_3edge[x][y][z]) return false;
                }
#endif   
        
#ifdef G_COLORED_3EDGES
        for (int x = 0; x < m_vertices; x++)
            for (int y = x+1; y < m_vertices; y++)
                for (int z = y+1; z < m_vertices; z++)
                {
                    if (m_colored_3edge[perm[x]][perm[y]][perm[z]] != h.m_colored_3edge[x][y][z]) return false;
                }
#endif   
        
        
#ifdef G_4EDGES
        for (int x = 0; x < m_vertices; x++)
            for (int y = x+1; y < m_vertices; y++)
                for (int z = y+1; z < m_vertices; z++)
                    for (int w = z+1; w < m_vertices; w++)
                    {
                        if(m_4edge[perm[x]][perm[y]][perm[z]][perm[w]] != h.m_4edge[x][y][z][w]) return false;
                    }
#endif  
        
#ifdef G_COLORED_VERTICES
        for (int u = 0; u < m_vertices; u++)
        {
            if (m_color_vertex[perm[u]] == 0 ||  h.m_color_vertex[u] == 0) continue;
            if (m_color_vertex[perm[u]] != h.m_color_vertex[u]) return false;
        }
#endif
        return true;
    }
    
    // Real isomorphism testing
	bool make_perm_isomorphic(int v, int *perm, bool *used, const flag &h) const
	{
		// Test if the permutation is isomorphism
		if (v >= m_vertices)
		{
#ifndef G_BE_BRAVE_AND_IGNORE_SAFETY_ASSERTS
            assert(is_mapping_an_equality(perm,h) == true);
#endif
            
			return true;
		}

		for (int pv = m_Theta; pv < m_vertices; pv++)
		{
			// check if used
			if (used[pv]) continue;

			if (!is_map_up_to_v_correct(v, pv, perm, h)) continue;

            
			perm[v] = pv;
			used[pv]  = true;
			
			if (make_perm_isomorphic(v+1, perm, used, h)) return true;
			
			used[pv]  = false;
		}
		
		return false;
	}
	
	
	bool is_isomorphic_to_not_colorblind(const flag &h) const
	{
        //if (labeled_vertices_cnt() > 0)
        //    cerr << "Starting testing not colorblind " << print() <<" and "<< h.print() << endl;

        
        // quick check
        if (m_vertices != h.m_vertices) 
        {
        if (labeled_vertices_cnt() > 0)
            //cerr << "Wrong number of vertices " << endl;
            return false;
        }
        if (labeled_vertices_cnt() != h.labeled_vertices_cnt())
        {
        if (labeled_vertices_cnt() > 0)
            //cerr << "Wrong number of labeled vertices " << labeled_vertices_cnt() << " " << h.labeled_vertices_cnt() << endl;
            return false;
        }
        
		
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM 		
		assert (!m_minlex_signature.empty() && !h.m_minlex_signature.empty());
		if (m_minlex_signature.compare(h.m_minlex_signature) != 0) return false;
		return true;
#endif		

#ifdef G_COLORED_VERTICES
        for (int c = 1; c < COLORS_VERTICES; c++)
        {
            //cerr << "c=" << c << "   "  << m_colored_vertices[c] << " "  << m_colored_vertices[0] << " " << h.m_colored_vertices[c] << " "  << h.m_colored_vertices[0] << endl;
            
	        if (m_colored_vertices[c] > h.m_colored_vertices[c]+h.m_colored_vertices[0]) return false;
            if (m_colored_vertices[c]+m_colored_vertices[0] < h.m_colored_vertices[c]) return false;
	    }
#endif
		
#ifdef G_COLORED_EDGES
        for (int c = 0; c < COLORS_EDGES; c++)
        {
            //if (m_vertices==0)
            //    cerr << c << "  " << m_colored_edges[c] << " " << h.m_colored_edges[c] << endl;
	        if (m_colored_edges[c] >  h.m_colored_edges[c]+h.m_colored_edges[0]) return false;
            if (m_colored_edges[c]+h.m_colored_edges[0] <  h.m_colored_edges[c]) return false;
	    }
#endif

#ifdef G_3EDGES
        if (m_3edges_cnt != h.m_3edges_cnt) return false;
#endif   

#ifdef G_COLORED_3EDGES
        for (int c = 1; c < COLORS_3EDGES; c++)
        {
            if (m_colored_3edges[c] !=  h.m_colored_3edges[c]) return false;
        }
#endif        
        
#ifdef G_4EDGES
        if (m_4edges_cnt != h.m_4edges_cnt) return false;
#endif   
//        if (m_vertices == 0)
//            cerr << "Testing not colorblind " << print() <<" and "<< h.print() << endl;

        //cerr << "Testing not colorblind " << print() <<" and "<< h.print() << endl;


#ifdef G_ORDERED_VERTICES
        // if vertices are ordered, there is no chance to do anything
        if (m_Theta != h.m_Theta)
            return false;
        
        int perm[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
        return is_mapping_an_equality(perm, h);
#else
		// Check for same type
		if (!have_same_type_not_colorblind(h)) return false;
		
		// try check all permutations --- WASTING HERE!!!!!!!!
		int perm[V];
		bool used[V];

		// Theta is preserved by the mapping
        for (int i = 0; i < m_Theta; i++)
        {
            perm[i] = i;
            used[i] = true;
        }
        
        for (int i = m_Theta; i < V; i++)
        {
            perm[i] = 0;
            used[i] = false;
        }
		
        return make_perm_isomorphic(m_Theta, perm, used, h);
#endif
	}

    
    void create_minlex_signature_blind_3edges(const flag &h, string &lex_min) const
    {
#ifdef G_COLORED_3EDGES_BLIND
        flag F;
        for (int p = 0; p < (int)g_allowed_3edgecolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_3edge_colors(g_allowed_3edgecolor_permutations[p]);
            string Fprint = F.print("");
            if (Fprint.compare(lex_min) < 0)
            {
                lex_min = Fprint;
            }
        }
#else
        string Hprint = h.print("");
        if (Hprint.compare(lex_min) < 0)
        {
            lex_min = Hprint;
        }
#endif
    }
    
    void create_minlex_signature_blind_edges(const flag &h, string &lex_min) const
    {
#ifdef G_COLORED_EDGES_BLIND
        flag F;
        for (int p = 0; p < (int)g_allowed_edgecolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_edge_colors(g_allowed_edgecolor_permutations[p]);
            create_minlex_signature_blind_3edges(F, lex_min);            
        }
#else
        create_minlex_signature_blind_3edges(h, lex_min);
#endif
    }
    
    void create_minlex_signature_oriented_edges(const flag &h, string &lex_min) const
    {
#ifdef G_ORIENTED_EDGES_BLIND
        create_minlex_signature_blind_edges(h,lex_min);
        flag swapped;
        swapped = h;
        swapped.swap_orientation();
        create_minlex_signature_blind_edges(swapped,lex_min);
#else
        create_minlex_signature_blind_edges(h,lex_min);
#endif
    }
    
    
    void create_minlex_signature_blind_vertices(const flag &h, string &lex_min) const
    {
#ifdef G_COLORED_VERTICES_BLIND
        flag F;
        for (int p = 0; p < (int)g_allowed_vertexcolor_permutations.size(); p++)
        {
            F = h; 
            F.permute_vertex_colors(g_allowed_vertexcolor_permutations[p]);
            create_minlex_signature_oriented_edges(F, lex_min);
        }
#else
        create_minlex_signature_oriented_edges(h, lex_min);
#endif
    }

        
    void create_minlex_signature()
    {
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM
#ifdef G_ORDERED_VERTICES
        Not implemented
#endif
		int perm[V];
		for (int i = 0; i < V; i++) perm[i] = i;
		
		std::sort (perm,perm+V);
					
		//print_for_human();
		string lex_min = print("");
		//cout << "Finding lex min for " << lex_min << endl;
		
		
		flag this_permuted;
		do {
			this_permuted.as_subflag(*this, perm, m_vertices, m_Theta, false);
				
				
            create_minlex_signature_blind_vertices(this_permuted, lex_min);
/*            
            string perm_string = this_permuted.print("");
				if (perm_string.compare(lex_min) < 0)
				{
					lex_min = perm_string;
				}
 */
		} while ( std::next_permutation(perm+m_Theta,perm+m_vertices) );
		
    	m_minlex_signature = lex_min;
#endif    	
    }


    void get_type_subflag(flag &f) const
    {
#ifdef G_ORDERED_VERTICES
        f.as_subflag(*this,m_labeled_vector,m_labeled_vertices_cnt,0);
        f.make_all_vertices_labeled();
#else
        const int mapping[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
        f.as_subflag(*this,mapping,m_Theta,m_Theta);        
#endif
    }
    
    
    // removes two vertices
    void remove_vertices(int u, int v)
    {
        int mapping[V];
        
        int id = 0;
        for (int i = 0; i < m_vertices;i++)
        {
            if (i == u || i == v) continue;
            mapping[id++] = i;
        }
        
        flag h;
        h.as_subflag(*this, mapping, m_vertices-2, 0);
        
        *this = h;
    }
    
    
    void as_subflag(const flag &h,const int *mapping, int vertices, int theta, bool create_signature=true)
    {
        set_vertices_and_Theta(vertices,theta);
		
#ifdef G_COLORED_EDGES
        for (int x = 0; x < m_vertices-1; x++)
            for (int y = x+1; y < m_vertices; y++)
            {
                color_edge(x,y,h.m_color_edge[mapping[x]][mapping[y]]);
            }
#endif
        
#ifdef G_3EDGES
        for (int x = 0; x < m_vertices-2; x++)
            for (int y = x+1; y < m_vertices-1; y++)
                for (int z = y+1; z < m_vertices; z++)
                {
                    set_3edge(x,y,z,h.m_3edge[mapping[x]][mapping[y]][mapping[z]]);
                    
                }
#endif   

#ifdef G_COLORED_3EDGES
        for (int x = 0; x < m_vertices-2; x++)
            for (int y = x+1; y < m_vertices-1; y++)
                for (int z = y+1; z < m_vertices; z++)
                {
                    color_3edge(x,y,z,h.m_colored_3edge[mapping[x]][mapping[y]][mapping[z]]);
                    
                }
#endif           
        
#ifdef G_4EDGES
        for (int x = 0; x < m_vertices-3; x++)
            for (int y = x+1; y < m_vertices-2; y++)
                for (int z = y+1; z < m_vertices-1; z++)
                    for (int w = z+1; w < m_vertices; w++)
                {
                    set_4edge(x,y,z,w,h.m_4edge[mapping[x]][mapping[y]][mapping[z]][mapping[w]]);
                    
                }
#endif  
        
#ifdef G_COLORED_VERTICES
		// copy vertices
        for (int x = 0; x < m_vertices; x++)
			color_vertex(x, h.m_color_vertex[mapping[x]]);
#endif


#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM
		if (create_signature) create_minlex_signature();
#endif
    }
    

    void as_subflag_in_blowup(const flag &h,const int *mapping, int vertices, int theta, bool create_signature=true)
    {
        set_vertices_and_Theta(vertices,theta);
        
#ifdef G_COLORED_EDGES
        for (int x = 0; x < m_vertices-1; x++)
            for (int y = x+1; y < m_vertices; y++)
            {
                if (mapping[x] != mapping[y])
                {
                    color_edge(x,y,h.m_color_edge[mapping[x]][mapping[y]]);
                }
                else
                {
                    color_edge(x,y,G_BLOW_UP_COLOR_EDGES);
                }
            }
#endif
        
#ifdef G_3EDGES
        for (int x = 0; x < m_vertices-2; x++)
            for (int y = x+1; y < m_vertices-1; y++)
                for (int z = y+1; z < m_vertices; z++)
                {
                    if (mapping[x] != mapping[y] && mapping[x] != mapping[z] && mapping[z] != mapping[y])
                    {
                        set_3edge(x,y,z,h.m_3edge[mapping[x]][mapping[y]][mapping[z]]);
                    }
                    // else no edge
                }
#endif
        
#ifdef G_COLORED_3EDGES
        for (int x = 0; x < m_vertices-2; x++)
            for (int y = x+1; y < m_vertices-1; y++)
                for (int z = y+1; z < m_vertices; z++)
                {
                    if (mapping[x] != mapping[y] && mapping[x] != mapping[z] && mapping[z] != mapping[y])
                    {
                    color_3edge(x,y,z,h.m_colored_3edge[mapping[x]][mapping[y]][mapping[z]]);
                    }
                    else
                    {
                        color_3edge(x,y,z,G_BLOW_UP_COLOR_3EDGES);
                    }
                }
#endif
        
#ifdef G_4EDGES
        for (int x = 0; x < m_vertices-3; x++)
            for (int y = x+1; y < m_vertices-2; y++)
                for (int z = y+1; z < m_vertices-1; z++)
                    for (int w = z+1; w < m_vertices; w++)
                    {
                        if (mapping[x] != mapping[y] && mapping[x] != mapping[z] && mapping[z] != mapping[y] && mapping[w] != mapping[x] && mapping[w] != mapping[y] && mapping[w] != mapping[z])
                        set_4edge(x,y,z,w,h.m_4edge[mapping[x]][mapping[y]][mapping[z]][mapping[w]]);
                        
                    }
#endif
        
#ifdef G_COLORED_VERTICES
        // copy vertices
        for (int x = 0; x < m_vertices; x++)
            color_vertex(x, h.m_color_vertex[mapping[x]]);
#endif
        
        
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM
        if (create_signature) create_minlex_signature();
#endif
    }
    
    
        
    int e()
    {
        return (m_vertices*(m_vertices-1))/2;
    }
    
    int e3()
    {
        return (m_vertices*(m_vertices-1)*(m_vertices-2))/6;
    }	
	
	
    bool contains_as_subflag_map_rest(const flag &g, int* mapping, int next_to_map, int first_available) const
    {
        if (next_to_map >= g.m_vertices)
        {
            flag h;
            h.as_subflag(*this, mapping,g.m_vertices, g.m_Theta);
            
            //This may happen!!
            // assert(g.m_Theta == 0);
         
            //cerr << "Testing " << h.print() << " and " << g.print() << endl;

            
            if (h.is_isomorphic_to(g))
            {
                return true;
            }
                         
            return false;
        }
        
#ifndef G_ORDERED_VERTICES
        if (first_available < g.m_Theta)
        {
            mapping[next_to_map] = first_available;
            if (contains_as_subflag_map_rest(g,mapping,next_to_map+1,first_available+1)) return true;            
        }
        else
        {
#endif
        for (int i = first_available; i < m_vertices; i++)
        {
            mapping[next_to_map] = i;
            if (contains_as_subflag_map_rest(g,mapping,next_to_map+1,i+1)) return true;
        }
#ifndef G_ORDERED_VERTICES
        }
#endif
        
        return false;
    }
    
    // Count density of g as a subflags
    bool contains_as_subflag(const flag &g) const
    {
        if (g.m_vertices > m_vertices) return false;
        
        int mapping[g.m_vertices];
                
        return contains_as_subflag_map_rest(g,mapping,0,0);        
    }
    
    
	void density_subflag_map_rest(const flag &g, int &total, int &good, int* mapping, int next_to_map, int first_available) const
	{
		if (next_to_map >= g.m_vertices)
		{
			flag h;
			h.as_subflag(*this, mapping,g.m_vertices, g.m_Theta);
			
			assert(g.m_Theta == 0);
			
			if (h.is_isomorphic_to(g)) good++;
			
			total++;
			return;
		}
		
		for (int i = first_available; i < m_vertices; i++)
		{
			mapping[next_to_map] = i;
			density_subflag_map_rest(g,total,good,mapping,next_to_map+1,i+1);
		}
	}
	
	// Count density of g as a subflags
    double density_subflag(const flag &g) const
    {
		if (g.m_vertices > m_vertices) return 0;
		
		int mapping[g.m_vertices];
		
		int total = 0;
		int good = 0;
		
		density_subflag_map_rest(g,total,good,mapping,0,0);
		
        return (double)good/(double)total;
	}

	int count_subflag(const flag &g) const
	{
		if (g.m_vertices > m_vertices) return 0;
		
		int mapping[g.m_vertices];
		
		int total = 0;
		int good = 0;
		
		density_subflag_map_rest(g,total,good,mapping,0,0);
		
      return good;
	}
    

    
    
    
    
    void count_labeled_copies_of_map_rest(const flag &g_labeled, int &good_maps, int* mapping, int *used, int next_to_map) const
    {
        if (next_to_map >= g_labeled.m_vertices)
        {
            flag h;
            h.as_subflag(*this, mapping,g_labeled.m_vertices, g_labeled.m_Theta);
            
            assert(g_labeled.m_Theta == g_labeled.m_vertices);
            
            if (h.have_same_type(g_labeled)) good_maps++;

            return;
        }
        
        for (int i = 0; i < m_vertices; i++)
        {
            if (used[i]) continue;
            mapping[next_to_map] = i;
            used[i] = 1;
            count_labeled_copies_of_map_rest(g_labeled,good_maps,mapping,used,next_to_map+1);
            used[i] = 0;
        }
    }
    

    int count_labeled_copies_of(const flag &g) const
    {
        if (g.m_vertices > m_vertices) return 0;
       
        // This does not work correctly for unlabeled graphs
        assert(g.m_Theta == 0 && m_Theta == 0);
        
        
        // We use have_same_type to check correct mappings...
        flag g_labeled = g;
        g_labeled.m_Theta = g.m_vertices;
        
        int mapping[g.m_vertices];
        int used[m_vertices];
        for (int i = 0; i < m_vertices; i++)
        {
            used[i] = 0;
        }
        
        int good_maps = 0;
        
        count_labeled_copies_of_map_rest(g_labeled,good_maps,mapping,used,0);
        
        return good_maps;
    }
    
    
    
    void generate_subflags_of_size_n_rest(int n, vector<flag> &subflags, int* mapping, int next_to_map, int first_available, const vector<int> &available_to_map) const
    {
        if (next_to_map >= n)
        {
            flag h;
            h.as_subflag(*this, mapping, n, 0);
            bool h_is_new = true;
            for (int i = 0; i < (int)subflags.size(); ++i)
            {
                if (h.is_isomorphic_to(subflags[i])) 
                {
                    h_is_new = false;
                    break;
                } 
            }
            if (h_is_new) subflags.push_back(h);            
            return;
        }
        
        for (int i = first_available; i < (int)available_to_map.size()-(n-next_to_map); i++)
        {
            mapping[next_to_map] = available_to_map[i];
            generate_subflags_of_size_n_rest(n,subflags,mapping,next_to_map+1,i+1,available_to_map);
        }
    }
   
    
    // Could be written as a template with size of in_subflag - should be faster
    // an/or with arrays
    // if in_subrgaph is specified, then only subflags containing in_subrgaph are generated
    int generate_subflags_of_size_n(int n, vector<flag> &subflags, const vector<int> &in_subflag = vector<int>()) const
    {
        if (n > m_vertices) return 0;
        
        // This does not work correctly for unlabeled graphs
        assert(m_Theta == 0);
                
        int mapping[n];
        
        vector<int> available_to_map;
        for (int i = 0; i < m_vertices; i++)
            if (find(in_subflag.begin(), in_subflag.end(), i) == in_subflag.end()) 
                available_to_map.push_back(i);
        
        for (int i = 0; i < (int)in_subflag.size(); i++)
            mapping[i] = in_subflag[i];
        
        generate_subflags_of_size_n_rest(n, subflags, mapping, (int)in_subflag.size(),0,in_subflag);
        
        return (int)subflags.size();
    }
    
    
    
    
    // Copy data from g if the number of vertices of g is <= current number
    void copy_from(const flag &g)
    {
        assert(m_vertices >= g.m_vertices);
     
#ifdef G_COLORED_VERTICES
        // copy vertices
        for (int x = 0; x < g.m_vertices; x++)
            color_vertex(x, g.m_color_vertex[x]);
#endif        
        
#ifdef G_3EDGES
        for (int x = 0; x < g.m_vertices-2; x++)
            for (int y = x+1; y < g.m_vertices-1; y++)
                for (int z = y+1; z < g.m_vertices; z++)
                {
                    set_3edge(x,y,z,g.m_3edge[x][y][z]);                    
                }
#endif

#ifdef G_COLORED_3EDGES
        for (int x = 0; x < g.m_vertices-2; x++)
            for (int y = x+1; y < g.m_vertices-1; y++)
                for (int z = y+1; z < g.m_vertices; z++)
                {
                    color_3edge(x,y,z,g.m_colored_3edge[x][y][z]);                    
                }
#endif        
        
#ifdef G_4EDGES
        for (int x = 0; x < g.m_vertices-3; x++)
            for (int y = x+1; y < g.m_vertices-2; y++)
                for (int z = y+1; z < g.m_vertices-1; z++)
                    for (int w = z+1; w < g.m_vertices; w++)
                {
                    set_4edge(x,y,z,w,g.m_4edge[x][y][z][w]);                    
                }
#endif        
        
#ifdef G_COLORED_EDGES
        for (int x = 0; x < g.m_vertices-1; x++)
            for (int y = x+1; y < g.m_vertices; y++)
            {
                color_edge(x,y,g.m_color_edge[x][y]);
            }        
#endif        
    }
    
/***************************************************************************************************************** Printing ****************/
	void print_for_human() const
	{
		// Prints the number of vertices, zero as not type and then part of the diagonal matrix
		cout << m_vertices << " " << m_Theta << " ";
		
#ifdef G_COLORED_VERTICES
		print_vertex_coloring(cout, " ");
		cout << " ";
#endif
		
		cout << endl;
	}

 
	
	string print(const string &delimeter=" ") const
	{
		stringstream ss;
		
		// Prints the number of vertices, zero as not type and then part of the diagonal matrix
		ss << m_vertices << delimeter << m_Theta << delimeter << delimeter;
#ifdef G_COLORED_VERTICES
		print_vertex_coloring(ss, delimeter);
		ss << delimeter;
#endif

#ifdef G_COLORED_EDGES
  		for (int u = 0; u < m_vertices; u++)
  		{
    		for (int v = u+1; v < m_vertices; v++)
				ss << delimeter << m_color_edge[u][v];
            ss << delimeter;
		}
#endif

#ifdef G_3EDGES
        ss << delimeter << delimeter << m_3edges_cnt;
        for (int x = 0; x < m_vertices-2; x++)
            for (int y = x+1; y < m_vertices-1; y++)
                for (int z = y+1; z < m_vertices; z++)
                {
                    if (m_3edge[x][y][z]) ss << delimeter << (char)('1'+x) << (char)('1'+y) << (char)('1'+z);
                }
        ss << delimeter << delimeter;
#endif   

#ifdef G_COLORED_3EDGES
#ifdef G_COLORED_3EDGES_PRINT_HUMAN
        int colored_3edges_count = 0;
        for (int c = 1; c < COLORS_3EDGES; c++)
        {
            colored_3edges_count += m_colored_3edges[c];
        }
        ss << delimeter << delimeter << colored_3edges_count;
        for (int x = 0; x < m_vertices-2; x++)
            for (int y = x+1; y < m_vertices-1; y++)
                for (int z = y+1; z < m_vertices; z++)
                {
                    if (m_colored_3edge[x][y][z] != 0)
                        ss << delimeter << (char)('1'+x) << (char)('1'+y) << (char)('1'+z) << '.' << (char)('0'+m_colored_3edge[x][y][z]);
                }
        ss << delimeter << delimeter;
        
#else        
        ss << delimeter << delimeter;
        ss << "-1" << delimeter;
        for (int x = 0; x < m_vertices-2; x++)
        {
            for (int y = x+1; y < m_vertices-1; y++)
            {
                for (int z = y+1; z < m_vertices; z++)
                {
                    ss << delimeter << m_colored_3edge[x][y][z];
                }
                ss << delimeter;
            }
            ss << delimeter;
        }
#endif        
#endif          
        
#ifdef G_4EDGES
        ss << delimeter << delimeter << m_4edges_cnt;
        for (int x = 0; x < m_vertices-3; x++)
            for (int y = x+1; y < m_vertices-2; y++)
                for (int z = y+1; z < m_vertices-1; z++)
                    for (int w = z+1; w < m_vertices; w++)
                {
                    if (m_4edge[x][y][z][w]) ss << delimeter << (char)('1'+x) << (char)('1'+y) << (char)('1'+z)<< (char)('1'+w);
                }
        ss << delimeter << delimeter;
#endif          
        
	
		return ss.str();
	}
    
#ifdef G_COLORED_VERTICES
	void print_vertex_coloring(ostream &ostr, const string &delimiter) const
	{
		for (int i = 0; i < m_vertices; i++)
		{
			ostr << delimiter << m_color_vertex[i];
		}
	}
#endif
	
/***************************************************************************************************************** Loading ****************/	
	void load_from_string(const char *str)
	{
		stringstream s(str);
		load_from_stream(s, -1, -1);
	}
		
	
   bool load_from_stream(istream &stream, int assumed_vertices, int assumed_theta)
   {
		int vertices, theta;
		stream >> vertices >> theta;
		if (!stream.good()) return false; 

#ifdef G_VERBOSE_OUTPUT
        if (assumed_theta != -1 && theta != assumed_theta)
        {
            cerr << "Err: assumed theta " << assumed_theta << " but loaded " <<  theta << " for " << vertices << " vertices " << endl;
        }
#endif       
		assert(assumed_theta == -1 || theta == assumed_theta);
        assert(assumed_vertices == -1 || assumed_vertices == vertices);
		set_vertices_and_Theta(vertices, theta);
		
#ifdef G_COLORED_VERTICES		
		for (int i = 0; i < m_vertices; i++)
		{
			int color_v;
			stream >> color_v;
			color_vertex(i,color_v);
		}
#endif		
		
#ifdef G_COLORED_EDGES
		int color;
		for (int u = 0; u < vertices; u++)
		{
			for (int v = u+1; v < vertices; v++)
			{
				stream >> color;
				color_edge(u,v,color);
			}
		}
#endif
       
       
#ifdef G_3EDGES
       int entries_3edge;
       stream >> entries_3edge;
       string str_3edge;
       for (int i = 0; i < entries_3edge; i++)
       {
           stream >> str_3edge;
           set_3edge(str_3edge[0]-'1', str_3edge[1]-'1', str_3edge[2]-'1', 1);
       }
#endif   

#ifdef G_COLORED_3EDGES
       // Format for 3 edges: 1) adjacency matrix, 2)  xyz.c
       int entries_colored_3edges;

       stream >> entries_colored_3edges;

       if (entries_colored_3edges == -1)
       {
           int color;
           for (int x = 0; x < m_vertices-2; x++)
               for (int y = x+1; y < m_vertices-1; y++)
                   for (int z = y+1; z < m_vertices; z++)
                   {
                       stream >> color;
                       color_3edge(x,y,z, color);
                   }
       }
       else
       {
           string str_3edge;
           for (int i = 0; i < entries_colored_3edges; i++)
           {
               stream >> str_3edge;
               assert(str_3edge.size() == 5);
               color_3edge(str_3edge[0]-'1', str_3edge[1]-'1', str_3edge[2]-'1', str_3edge[4]-'0');
           }           
       }
#endif       
       
#ifdef G_4EDGES
       int entries_4edge;
       stream >> entries_4edge;
       string str_4edge;
       for (int i = 0; i < entries_4edge; i++)
       {
           stream >> str_4edge;
           set_4edge(str_4edge[0]-'1', str_4edge[1]-'1', str_4edge[2]-'1', str_4edge[3]-'1', 1);
       }
#endif          
       //cerr << print() << endl;
       
       create_minlex_signature();

		return true;
	}
	
    

public:
    
    flag& operator=(const flag& f)
    {
        m_vertices = f.m_vertices;
        m_Theta    = f.m_Theta;

#ifdef G_ORDERED_VERTICES
        m_labeled_vertices_cnt = f.m_labeled_vertices_cnt;
        for (int i = 0; i < m_vertices; i++)
        {
            m_labeled[i] = f.m_labeled[i];
            m_labeled_vector[i] = f.m_labeled_vector[i]; 
        }
#endif
        
#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM                
        m_minlex_signature = f.m_minlex_signature;
#endif        
        
#ifdef G_COLORED_VERTICES
        
        for (int i = 0; i < m_vertices; i++)
        {
            m_color_vertex[i] = f.m_color_vertex[i];
        }
        
        //memcpy(m_color_vertex, f.m_color_vertex, sizeof(int)*m_vertices);
        
        for (int i = 0; i < COLORS_VERTICES; i++)
        {
            m_colored_vertices[i] = f.m_colored_vertices[i];
        }        
#endif
        
#ifdef G_COLORED_EDGES
        for (int i = 0; i < m_vertices; i++)
            for (int j = 0; j < m_vertices; j++)
                {
                    m_color_edge[i][j] = f.m_color_edge[i][j];
                }
        
        for (int i = 0; i < COLORS_EDGES; i++)
        {
            m_colored_edges[i] = f.m_colored_edges[i];
        }         
        
#endif
        
        
#ifdef G_3EDGES
        for (int i = 0; i < m_vertices; i++)
            for (int j = 0; j < m_vertices; j++)
                for (int k = 0; k < m_vertices; k++)
                {
                    m_3edge[i][j][k] = f.m_3edge[i][j][k];
                }
        m_3edges_cnt = f.m_3edges_cnt;
#endif

#ifdef G_COLORED_3EDGES
        for (int i = 0; i < m_vertices; i++)
            for (int j = 0; j < m_vertices; j++)
                for (int k = 0; k < m_vertices; k++)
        {
            m_colored_3edge[i][j][k] = f.m_colored_3edge[i][j][k];
        }
        
        for (int i = 0; i < COLORS_3EDGES; i++)
        {
            m_colored_3edges[i] = f.m_colored_3edges[i];
        }         
        //memcpy(m_colored_3edges, f.m_colored_3edges, sizeof(int)*COLORS_3EDGES);

#endif        
#ifdef G_4EDGES
        TODO!!
        int m_4edge[V][V][V][V]; // m_4edges[x][y][v][w] == 1 it it is an edge
        int m_4edges_cnt; // number of 4-edges (for faster isomorphism and so on)
#endif 
        return *this;
    }
    
		
	int m_vertices;  // number of vertices
    int m_Theta;     // number of labeled vertices in case of not ordered. Binary string which are labeled in ordered
 	
#ifdef G_ORDERED_VERTICES 
    int  m_labeled_vertices_cnt; // number of labeled vertices
    bool m_labeled[V];           // every vertex has true/false if labeled
    int  m_labeled_vector[V];    // ordered list of labeled vertices
#endif
//	int m_id; // id of the original flag

#ifdef G_USE_LEXMIN_FOR_ISOMORPHISM            
// for fast isomorphism testing
	string m_minlex_signature;
#endif
    
	
#ifdef G_COLORED_VERTICES
	int m_color_vertex[V]; // color of a vertex (for bipartite complete flags?)
	int m_colored_vertices[COLORS_VERTICES];
#endif
	
#ifdef G_COLORED_EDGES
	int m_color_edge[V][V];  // adjacency matrix
//	int m_colored_deg[V][COLORS_EDGES]; // number of edges from each vertex of a particular color
	int m_colored_edges[COLORS_EDGES]; // number of edges of each color
//	vector<int> m_deg_sorted;
//	bool m_deg_sorted_valid;
#endif

    
#ifdef G_3EDGES
    int m_3edge[V][V][V]; // m_3edges[x][y][v] == 1 it it is an edge
    int m_3edges_cnt; // number of 3-edges (for faster isomorphism and so on)
#endif
#ifdef G_COLORED_3EDGES
    int m_colored_3edge[V][V][V]; // m_3edges[x][y][v] == color
    int m_colored_3edges[COLORS_3EDGES]; // number of 3-edges for each of the colors
#endif    
    
#ifdef G_4EDGES
    int m_4edge[V][V][V][V]; // m_4edges[x][y][v][w] == 1 it it is an edge
    int m_4edges_cnt; // number of 4-edges (for faster isomorphism and so on)
#endif    
};


class flag_and_coefficient
{
public:
	flag   g;
	double coefficient;
};


// Try if the number should be actuallly rounded
double smart_round(double d)
{
    double dr = round(d);
    if (abs(dr-d) < 0.00000001) return dr;
    return d;
}


vector<flag> g_unlabeled_flags[V+1];
vector<flag> g_types[V+1]; // types - it is just same as g_unlabeled_flags, but all vertices are labeled :-) Used for reduced types
vector<vector<flag> >  g_flags; // flags to process using multiplication - every type&size is a separate list

vector<flag_and_coefficient> g_objective_combination;
vector<flag> g_forbidden_subflags;
vector<flag> g_forbidden_subflags_by_size[V+1];
string       g_program_description;  // loaded from the objective function


#ifdef G_USE_PERMITTED_SUBFLAGS
vector<flag>  g_permitted_subflags[V+1];
#endif


int binomial(int n, int k) {
    int b = 1;
    for (int i = 0; i < k; ++i) {
        b *= (n - i);
        b /= (i + 1);
    }
    return b;
}

int factorial(int n) {
    int b = 1;
    for (int i = 2; i < n; i++) {
        b *= i;
    }
    return b;
}



string filename_prefix()
{
    stringstream filename;
    
    filename << "F"
#ifdef G_COLORED_VERTICES
    << "_vertices" << COLORS_VERTICES-1
#ifdef G_ORDERED_VERTICES
    << "ordered"
#endif
#else
#ifdef G_ORDERED_VERTICES
    << "_ordered"
#endif
#endif
#ifdef G_COLORED_VERTICES_BLIND
    << "blind"
#endif
#ifdef G_COLORED_EDGES
    << "_edges" << COLORS_EDGES-1
#endif
#ifdef G_COLORED_EDGES_BLIND
    << "blind"
#endif
#ifdef G_ORIENTED_EDGES
    << "oriented"
#endif
#ifdef G_ORIENTED_EDGES_BLIND
    << "blind"
#endif
#ifdef G_3EDGES
    << "_3edges"
#endif
#ifdef G_COLORED_3EDGES
    << "_3edges" << COLORS_3EDGES-1
#endif
#ifdef G_COLORED_3EDGES_BLIND
    << "blind"
#endif    
#ifdef G_4EDGES
    << "_4edges"
#endif
    ;
    return filename.str();
}




inline int find_flag_in_list(const flag &f, const vector<flag> &fv)
{
    for (int i = 0; i < (int)fv.size(); ++i)
    {
        if (f.is_isomorphic_to(fv[i]))
        {
            return i;
        }
    }
    return -1;
}

inline int get_flag_type_in_list(const flag& f, vector< vector<flag> > &flags)
{
#ifdef USE_REDUCED_TYPES
    const int identity[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30};
    flag f_type;
    f_type.as_subflag(f,identity,f.m_Theta,0);
#endif 
    
    // Find correct type
    for (int i = 0; i < (int)flags.size(); i++)
    {
        if (flags[i].size() == 0)
        {
            cerr << "Some bug" << endl;
        }
        if (f.have_same_type(flags[i][0])) return i;
#ifdef USE_REDUCED_TYPES
        if (f.m_Theta == flags[i][0].m_Theta)
        {
            flag g_flags_type;
            g_flags_type.as_subflag(flags[i][0],identity,flags[i][0].m_Theta,0);
            if (g_flags_type.is_isomorphic_to(f_type))
                return -2;
        }
#endif        
        
    }
    
    return -1;
}

inline void include_flag_in_list(const flag& f, vector< vector<flag> > &flags, int id = -3, bool ignore_reduced_types = true)
{
    if (id == -3)
    {
        id = get_flag_type_in_list(f, flags);
    }
#ifdef  USE_REDUCED_TYPES   
    if (id == -2 && ignore_reduced_types) return;
#endif   
    if (id == -1 || id == -2)
    {
        vector<flag> new_type_list;
        new_type_list.push_back(f);
        flags.push_back(new_type_list);
        return;
    }
    
    flags[id].push_back(f);
}


inline void include_flag_in_list_if_new(const flag& f, vector< vector<flag> > &flags, bool ignore_reduced_types = true)
{
    int id = get_flag_type_in_list(f, flags);
#ifdef USE_REDUCED_TYPES
    if (id == -2 && ignore_reduced_types) 
    {
        return;
    }
#endif
    if (id == -1 || id == -2)
    {
//        cerr << "new type" << endl;
        include_flag_in_list(f, flags, id,ignore_reduced_types);
        return;
    }
    
    if (find_flag_in_list(f,flags[id]) != -1)
    {
//        cerr << "duplicate" << endl;
        return;   
    }
//    cerr << "new for known type" << f.print() << endl;
    flags[id].push_back(f);
}





inline bool is_flag_forbidden(const flag &g)
{
    for (int i = 0; i < (int)g_forbidden_subflags.size(); i++)
    {
        //cerr << "Testing " << g.print() << " and " << g_forbidden_subflags[i].print() << endl;
        if (g.contains_as_subflag(g_forbidden_subflags[i])) 
        {
            //cerr << "Forbidding " << g.print() << " because of " << g_forbidden_subflags[i].print() << endl;
            return true;
        }
    }
#ifdef G_USE_PERMITTED_SUBFLAGS
    vector<flag> subflags_list;
    for (int i = 1; i <= g.m_vertices; i++)
    {
        if (g_permitted_subflags[i].size() == 0) continue;
        
        subflags_list.clear();
        g.generate_subflags_of_size_n(i, subflags_list);
        
        //if (subflags_list.size() > 1)
        // cerr << "Number of subflags is " << subflags_list.size() << endl;
        
        // Certainly there is one not permitted...
        if (subflags_list.size() > g_permitted_subflags[i].size()) return true;
        
        for (int j = 0; j < (int)subflags_list.size(); j++)
        {
            if (find_flag_in_list(subflags_list[j], g_permitted_subflags[i]) == -1)
                return true;
        }
    }
#endif    
    return false;
}

//#ifdef G_USE_PERMITTED_SUBFLAGS  not used in early generation - maybe should be... ?
inline bool is_flag_forbidden_avoid_zeros(const flag &g, const vector<int> &in_subrgaph = vector<int>())
{
    vector<flag> subflags_list;
    for (int i = 1; i <= g.m_vertices; i++)
    {
        //if (g_permitted_subflags[i].size() == 0) continue;        
        if (g_forbidden_subflags_by_size[i].size() == 0) continue;
        
        subflags_list.clear();
        g.generate_subflags_of_size_n(i, subflags_list, in_subrgaph);
                
        for (int j = 0; j < (int)subflags_list.size(); j++)
        {
// Do not check subgraphs that are not fully colored
#ifdef G_COLORED_VERTICES
            if (subflags_list[j].m_colored_vertices[0] > 0) continue;
#endif
#ifdef G_COLORED_EDGES            
            if (subflags_list[j].m_colored_edges[0] > 0) continue;
#endif            
#ifdef G_COLORED_3EDGES            
            if (subflags_list[j].m_colored_3edges[0] > 0) continue;
#endif            
            if (find_flag_in_list(subflags_list[j], g_forbidden_subflags_by_size[i]) != -1) return true;
        }
    }
    return false;
}

/*
inline bool is_flag_forbidden_noninduced(const flag &g)
{
    for (int i = 0; i < (int)g_forbidden_subflags.size(); i++)
    {
        if (g.has_as_notinduced_subflag(g_forbidden_subflags[i])) return true;
    }
    return false;
}
*/

inline bool g_already_in_known_flags(flag &g, vector<flag> &flag_list)
{
    for (unsigned int i = 0; i < flag_list.size(); i++)
    {
        if (flag_list[i].is_isomorphic_to(g))
        {
            return true;
        }
    }
    return false;
}

inline void add_g_to_known_flags(flag &g, vector<flag> &flag_list)
{
    flag_list.push_back(g);
    //if (g.m_3edges_cnt >= 20)
    {
    //    cerr << g.print() << endl;
    }
}


#ifdef G_COLORED_EDGES
void try_color_edge(flag &g, int u, int v, vector<flag> &flag_list)
{
    if (v >= g.m_vertices || v == u)
    {
        u++;
        v = u+1;
#ifdef G_EARLY_FORBIDDEN_TEST_IN_GENERATING
        // test if graph on vertices 0...u is not forbidden
        const int identity[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
        flag g_to_u;
        g_to_u.as_subflag(g,identity, u+1, 0);
        
        cerr << "Testing " << g_to_u.print() << endl;
        
        if (is_flag_forbidden(g_to_u)) return;
#endif
    }
    if (v >= g.m_vertices)
    {
        if (is_flag_forbidden(g)) return;
        
        g.create_minlex_signature();
        
        if (!g_already_in_known_flags(g,flag_list)) add_g_to_known_flags(g,flag_list);
        //		add_g_to_known_flags(flagh_list);
        return;
    }
    
    
    // Color only uncolored edges.....
    if (g.m_color_edge[u][v] == 0)
    {
// Either color 1 us used for blow-up or for tournaments it is not used at all
// since color 1 does not change orientation
#if defined(G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY) || defined(G_TOURNAMENT)         
        for (int color = 2; color < COLORS_EDGES; color++)
#else
        for (int color = 1; color < COLORS_EDGES; color++)
#endif
        {
            g.color_edge(u,v,color);
            try_color_edge(g,u,v+1,flag_list);
#ifdef G_ORIENTED_EDGES
            if (color > 1)
            {
                g.color_edge(v,u,color);
                try_color_edge(g,u,v+1,flag_list);
            }
#endif            
        }
        g.color_edge(u,v,0);
    }
    else
    {
        try_color_edge(g,u,v+1,flag_list);
    }
}
#endif


#ifdef G_3EDGES 
void try_add_3edges(flag &g, int x, int y, int z, vector<flag> &flag_list)
{
    if (z >= g.m_vertices || z == x)
    {
        y++;
        if (y >= g.m_vertices-1)
        {
            x++;
            if (x >= g.m_vertices-2)
            {
                if (is_flag_forbidden(g)) return; 

                g.create_minlex_signature();
                
                if (!g_already_in_known_flags(g,flag_list)) add_g_to_known_flags(g,flag_list);
                return;
            }
            y = x+1;
        }
        z = y+1;
        try_add_3edges(g,x,y,z,flag_list);
        return;
    }
    
    if (g.m_3edge[x][y][z] != 0) try_add_3edges(g,x,y,z+1,flag_list);
    else
    {
        g.set_3edge(x,y,z,1);
        try_add_3edges(g,x,y,z+1,flag_list);
        g.set_3edge(x,y,z,0);
        try_add_3edges(g,x,y,z+1,flag_list);
    }
}
#endif

#ifdef G_COLORED_3EDGES 
void try_color_3edges(flag &g, int x, int y, int z, vector<flag> &flag_list)
{
    if (z >= g.m_vertices || z == x)
    {
        y++;
        if (y >= g.m_vertices-1)
        {
            x++;
            if (x >= g.m_vertices-2)
            {
//#ifndef G_EARLY_FORBIDDEN_TEST_IN_GENERATING
                if (is_flag_forbidden(g)) return; 
//#endif
                if (!g_already_in_known_flags(g, flag_list)) 
                {
                    add_g_to_known_flags(g,flag_list);
                }
                //cerr << "FOUND " << endl << g.print() << endl;
                return;
            }
            y = x+1;
        }
        z = y+1;
        try_color_3edges(g,x,y,z,flag_list);
        return;
    }
    
    if (g.m_colored_3edge[x][y][z] != 0) try_color_3edges(g,x,y,z+1,flag_list);
    else
    {
#ifdef G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY
        for (int color = 2; color < COLORS_3EDGES; color++)
#else
        for (int color = 1; color < COLORS_3EDGES; color++)
#endif            
        {
            g.color_3edge(x,y,z,color);
#ifdef G_EARLY_FORBIDDEN_TEST_IN_GENERATING
            const vector<int> edge_xyz = {x,y,z};
            if (is_flag_forbidden_avoid_zeros(g,edge_xyz)) continue;
#endif
            try_color_3edges(g,x,y,z+1,flag_list);
        }
        g.color_3edge(x,y,z,0);
    }
}
#endif

#ifdef G_4EDGES 
void try_add_4edges(flag &g, int x, int y, int z, int w, vector<flag> &flag_list)
{
    if (w >= g.m_vertices || w == x)
    {
        z++;
        if (z >= g.m_vertices-1)
        {
            y++;
            if (y >= g.m_vertices-2)
            {
                x++;
                if (x >= g.m_vertices-3)
                {
                    if (is_flag_forbidden(g)) return;                
                    g.create_minlex_signature();
                    if (!g_already_in_known_flags(g,flag_list)) add_g_to_known_flags(g,flag_list);
                    return;
                }
                y = x+1;
            }
            z = y+1;
        }
        w = z+1;
        try_add_4edges(g,x,y,z,w,flag_list);
        return;
    }
    
    if (g.m_4edge[x][y][z][w] != 0) try_add_4edges(g,x,y,z,w+1,flag_list);
    else
    {
        g.set_4edge(x,y,z,w,1);
        try_add_4edges(g,x,y,z,w+1,flag_list);
        g.set_4edge(x,y,z,w,0);
        try_add_4edges(g,x,y,z,w+1,flag_list);
    }
}
#endif



void extensions_of_g_edges(flag &g, vector<flag> &flag_list)
{
    //cerr << "Generating extensions of " << g.print() << endl;
#ifdef G_3EDGES    
#ifdef G_COLORED_EDGES
#error I dont know how to generate 2edges and 3edges at the same time.
#endif
#endif
    
#ifdef G_3EDGES       
    try_add_3edges(g,0,1,2,flag_list);
    return;
#endif
#ifdef G_COLORED_3EDGES       
    try_color_3edges(g,0,1,2,flag_list);
    return;
#endif    
#ifdef G_4EDGES       
    try_add_4edges(g,0,1,2,3,flag_list);
    return;
#endif    
#ifdef G_COLORED_EDGES
    try_color_edge(g,0,1,flag_list);
    return;
#endif    
}


#ifdef G_COLORED_VERTICES
void try_color_vertex(flag &g, int u, vector<flag> &flag_list)
{
    if (u >= g.m_vertices)
    {
#if defined(G_COLORED_EDGES) || defined(G_COLORED_3EDGES) || defined(G_3EDGES) || defined(G_4EDGES)
        extensions_of_g_edges(g,flag_list);        
        //        try_color_edge(g, 0,1,flag_list);
#else
        if (is_flag_forbidden(g)) return;
        
        g.create_minlex_signature();
        
        if (!g_already_in_known_flags(g, flag_list)) add_g_to_known_flags(g, flag_list);
#endif
        return;
    }
    
    // Color only uncolored vertices.....
    if (g.m_color_vertex[u] == 0)
    {
        for (int color = 1; color < COLORS_VERTICES; color++)
            //        for (int color = 1; color < 3; color++)
        {
            g.color_vertex(u,color);
            try_color_vertex(g,u+1,flag_list);
        }
        g.color_vertex(u,0);
    }
    else
    {
        try_color_vertex(g,u+1,flag_list);
    }
}
#endif



void extensions_of_g(flag &g, vector<flag> &flag_list)
{
#ifdef G_COLORED_VERTICES
    try_color_vertex(g,0,flag_list);
#else
    extensions_of_g_edges(g,flag_list);
#endif
}



// u is original, v will be a copy
// edges containing both will have color 1
void duplicate_vertex_u_to_v(flag &g, int u, int v)
{
#if defined(G_4EDGES) || defined(G_3EDGES) 
    assert(0); // this is not implemented
#endif

#ifdef G_COLORED_EDGES
    
    for (int x = 0; x < g.m_vertices-1; x++)
    {
        if (x == u) 
            g.color_edge(v,x,1);
        else
            g.color_edge(v,x,g.m_color_edge[u][x]);
    }
#endif

#ifdef G_COLORED_3EDGES
    for (int x = 0; x < g.m_vertices-1; x++)
    {
        for (int y = x+1; y < g.m_vertices-1; y++)
        {
            if (x == u || y == u)
                g.color_3edge(v,x,y,1);
            else
                g.color_3edge(v,x,y,g.m_colored_3edge[u][x][y]);
        }
    }
#endif

#ifdef G_COLORED_VERTICES
    g.color_vertex(v,g.m_color_vertex[u]);    
#endif
    
}


#ifdef G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY
void generate_blowups_by_one_vertex(const vector<flag> &original_flags, vector<flag> &flag_list)
{
    for (int i = 0; i < (int)original_flags.size(); i++)
    {
        int v = original_flags[i].m_vertices;
        for (int u = 0; u < v; u++)
        {
            flag g;
            g.set_vertices(v+1);
            g.copy_from(original_flags[i]);
            duplicate_vertex_u_to_v(g, u, v);
            
            //if (is_flag_forbidden(g)) return;  // this is unlikely, but we need to try...
            if (!g_already_in_known_flags(g,flag_list)) add_g_to_known_flags(g,flag_list);
        }
    }
    
}
#endif

/*
 void generate_permutations(vector<flag> &flag_list, int vertices)
 {
 g.set_vertices(vertices);
 int perm[g.m_vertices];
 for (int i = 0; i < g.m_vertices; i++) perm[i] = i;
 
 do {
 
 g.set_vertices(g.m_vertices);
 for (int u = 0; u < g.m_vertices; u++)
 for (int v = u+1; v < g.m_vertices; v++)
 {
 if (perm[u]<perm[v]) g.color_edge(u,v,1);
 else g.color_edge(u,v,2);
 }
 
 if (!g_already_in_known_flags(flag_list)) add_g_to_known_flags(flag_list);
 
 
 } while ( std::next_permutation(perm,perm+g.m_vertices) );
 }
*/



///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////// P(F1,F2,H)


// Rest of F2 is also piskced as a subset
void pick_F2_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_F2, int next_to_map,  bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F2.m_vertices)
    {
        flag H_F2;
        H_F2.as_subflag(H, mapping_F2, F2.m_vertices, F2.m_Theta);
        if (!H_F2.is_isomorphic_to(F2)) return;
		
        good_maps++;
        
        return;
    }
    
    int min_vertex = 0;
    if (next_to_map > F2.m_Theta) min_vertex = mapping_F2[next_to_map-1]+1;
    
    for (int v = min_vertex; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        used_from_H[v] = true;
        mapping_F2[next_to_map] = v;
        pick_F2_mapping(F1,F2,H,mapping_F2,next_to_map+1,used_from_H,good_maps);
        used_from_H[v] = false;
    }
    
}

// Rest of F1 is picked only as a subset
void pick_F1_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_F1, int *mapping_F2, int next_to_map,  bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F1.m_vertices)
    {
		//cout << "." << endl;

        // Check if the mapping is corect!
        flag H_F1;
        H_F1.as_subflag(H, mapping_F1, F1.m_vertices,F1.m_Theta);
		
        if (!H_F1.is_isomorphic_to(F1)) return;
        
        pick_F2_mapping(F1,F2,H,mapping_F2,F2.m_Theta,used_from_H,good_maps);
        return;
    }
    
    int min_vertex = 0;
    if (next_to_map > F1.m_Theta) min_vertex = mapping_F1[next_to_map-1]+1;
    
    for (int v = min_vertex; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        used_from_H[v] = true;
        mapping_F1[next_to_map] = v;
        pick_F1_mapping(F1,F2,H,mapping_F1,mapping_F2,next_to_map+1,used_from_H,good_maps);
        used_from_H[v] = false;
    }
}

// Theta mapping is picked as all permutations...
void pick_theta_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_theta, int next_to_map, bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F1.m_Theta)
    {
        int mapping_F1[F1.m_vertices];
        int mapping_F2[F2.m_vertices];
        for (int i = 0; i < F1.m_Theta; i++)
        {
            mapping_F2[i] = mapping_F1[i] = mapping_theta[i];
        }
				
        pick_F1_mapping(F1,F2,H,mapping_F1,mapping_F2, F1.m_Theta, used_from_H,good_maps);
        return;
    }
	
    for (int v = 0; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        if (!H.is_map_up_to_v_correct(next_to_map, v, mapping_theta, F1)) continue;
       
         used_from_H[v] = true;
         mapping_theta[next_to_map] = v;
         pick_theta_mapping(F1,F2,H,mapping_theta,next_to_map+1,used_from_H,good_maps);
		 used_from_H[v] = false;
    }
}

inline double P_F1_F2_IN_H(const flag &F1, const flag &F2, const flag &H)
{
    assert(F1.m_Theta == F2.m_Theta);

#if defined(G_COLORED_EDGES) && !defined(G_COLORED_EDGES_BLIND)
	for (int c = 1; c < COLORS_EDGES; c++)
	{
        //		cout << "e" << endl;
        if (F1.m_colored_edges[c] > H.m_colored_edges[c]) return 0;
        if (F2.m_colored_edges[c] > H.m_colored_edges[c]) return 0;
	}
#endif

    
#ifdef G_3EDGES    
    if (F1.m_3edges_cnt > H.m_3edges_cnt) return 0;
    if (F2.m_3edges_cnt > H.m_3edges_cnt) return 0;
#endif   

#if defined(G_COLORED_3EDGES) && !defined(G_COLORED_3EDGES_BLIND)
    for (int c = 1; c < COLORS_3EDGES; c++)
    {
        //		cout << "e" << endl;
        if (F1.m_colored_3edges[c] > H.m_colored_3edges[c]) return 0;
        if (F2.m_colored_3edges[c] > H.m_colored_3edges[c]) return 0;
    }
#endif
    
#ifdef G_4EDGES    
    if (F1.m_4edges_cnt > H.m_4edges_cnt) return 0;
    if (F2.m_4edges_cnt > H.m_4edges_cnt) return 0;
#endif   
    
#ifdef G_COLORED_VERTICES
#ifdef G_COLORED_VERTICES_BLIND
    assert(0);
#else 
	for (int c = 1; c < COLORS_VERTICES; c++)
	{
		//cout << "v " << F1.m_colored_vertices[c] << " " << H.m_colored_vertices[c] << endl;
		if (F1.m_colored_vertices[c] > H.m_colored_vertices[c]) return 0;
		if (F2.m_colored_vertices[c] > H.m_colored_vertices[c]) return 0;
	}
#endif    
#endif
	
		
    int mapping_theta[F1.m_Theta];
    bool used_from_H[H.m_vertices];
    
    for (int i = 0; i < H.m_vertices; i++)
    {
        used_from_H[i] = false;
    }
	
    int good_maps = 0;
    pick_theta_mapping(F1,F2,H,mapping_theta,0,used_from_H,good_maps);
	
    
    int all_maps = 1;
#ifndef WRITE_INTEGER_DENSITIES_FOR_FLAGS
    for (int i = 0; i < F1.m_Theta; i++) all_maps *= (H.m_vertices-i);
    all_maps *= binomial(H.m_vertices-F1.m_Theta,F1.m_vertices-F1.m_Theta);
    all_maps *= binomial(H.m_vertices-F1.m_vertices,F2.m_vertices-F1.m_Theta);
#endif
    
	//    if (good_maps > 0)
	//        cout << "good_maps/all_maps: " << good_maps << "/" << all_maps << endl;
    
    
    return (double)good_maps/(double)all_maps;
}



inline double P_F1_IN_H(const flag &F1, const flag &H, bool density=true)
{
    
    assert(F1.m_vertices <= H.m_vertices);
    
    flag type;
    F1.get_type_subflag(type);    

    if (H.labeled_vertices_cnt() > 0)
    {
        flag typeH;
        H.get_type_subflag(typeH);    
        if (!type.contains_as_subflag(typeH)) return 0;
    }    
    
    // Some easy checks...
#if defined(G_COLORED_EDGES) && !defined(G_COLORED_EDGES_BLIND)
        
    for (int c = 1; c < COLORS_EDGES; c++)
    {
        if (F1.m_colored_edges[c] > H.m_colored_edges[c]) return 0;
    }
#endif
    
    
#ifdef G_3EDGES
    if (F1.m_3edges_cnt > H.m_3edges_cnt) return 0;
#endif   
    
#if defined(G_COLORED_3EDGES) && !defined(G_COLORED_3EDGES_BLIND)
    for (int c = 1; c < COLORS_3EDGES; c++)
    {
        if (F1.m_colored_3edges[c] > H.m_colored_3edges[c]) return 0;
    }
#endif
    
#ifdef G_4EDGES    
    if (F1.m_4edges_cnt  > H.m_4edges_cnt) return 0;
#endif   
    
#if defined(G_COLORED_VERTICES) && !defined(G_COLORED_VERTICES_BLIND)
    for (int c = 1; c < COLORS_VERTICES; c++)
    {
        if (F1.m_colored_vertices[c] > H.m_colored_vertices[c]) return 0;
    }
#endif
    
#ifdef G_CROSSINGS	
    if (F1.m_crossings_cnt > H.m_crossings_cnt) return 0;
#endif
    
    
#ifdef G_ORDERED_VERTICES
    assert(0); // not implemented
#else
    
    int good_maps = 0;
    int all_maps = 0;
    
    int vertex_split[V];
    
    int t = 0;
    for (int i = 0; i < type.m_vertices; i++) vertex_split[t++] = i;
    for (int i = type.m_vertices; i < F1.m_vertices; i++) vertex_split[t++] = -1;
    for (;t < H.m_vertices; t++) vertex_split[t] = -4;
    
    std::sort (vertex_split+H.labeled_vertices_cnt(),vertex_split+H.m_vertices);
    
    flag tmp;
    int mapping[V];
    int next_to_map; // temporary
    do {
        all_maps++;
        
        
        // check if Theta correct
        for (int j = 0; j < H.m_vertices; j++)
        {
            if (vertex_split[j] < 0) continue;
            if (vertex_split[j] < type.m_vertices) mapping[vertex_split[j]] = j;
        }
        tmp.as_subflag(H,mapping,type.m_vertices,type.m_Theta);
        //cerr << tmp.print() << " vs " << type.print();
        if (!tmp.is_isomorphic_to(type)) continue;
        
        
        // check if F1 correct
        next_to_map = type.m_vertices;
        for (int j = 0; j < H.m_vertices; j++)  if (vertex_split[j] == -1) mapping[next_to_map++] = j;
        tmp.as_subflag(H,mapping,F1.m_vertices,type.m_Theta);
        if (!tmp.is_isomorphic_to(F1)) continue;
        
        good_maps++;
        
    } while ( std::next_permutation(vertex_split+H.labeled_vertices_cnt(),vertex_split+H.m_vertices) ); 
    
    //cerr << "Good/All: " << good_maps << "/" << all_maps << endl;
    
    if (density)
        return (double)good_maps/(double)all_maps;
    else
        return good_maps;
#endif        
}



bool load_flags_from_file(string filename, vector<flag> &flag_list)
{
    ifstream infile;
    infile.open (filename.c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file with flags " << filename << endl;
        return false;
    }
    
    cerr << "Loading labeled flags from file " << filename << endl;
    
    
    flag f;
    while (f.load_from_stream(infile,-1,-1))
    {
        flag_list.push_back(f);
    }
    
    infile.close();
    
    return true;    
}

bool dump_flags_to_file(string filename, vector<flag> &flag_list)
{
    ofstream outfile;
    outfile.open (filename, ofstream::out);
    if (!outfile.good())
    {
        cerr << "Failed opening file " << filename << endl;
        return false;
    }
    
    cerr << "Writing flags to file " << filename << endl;
    
    for (unsigned int x = 0; x < flag_list.size(); x++)
    {
        outfile << flag_list[x].print() << endl;
    }
    
    outfile.close();
    
    return true;
}


// Not paralel version - very slow and not completely correct - in particular for ordered version.
void generate_labeled_flags_of_one_type(int flag_size, const flag &type, vector<flag> &flag_list)
{
#ifdef G_ORDERED_VERTICES
    assert(0);
#endif
    
    // Saving of the flags to a separate file and/or loading them
    string filename = filename_prefix() + "_size"+to_string((long long)flag_size)+"_type"+ type.print("")+".txt";
    
    if (load_flags_from_file(filename,flag_list))
    {
        cerr << "Loaded " <<  flag_list.size() << " flags from " << filename << endl;
        return;
    }
    
    int to_label_size = (int)g_unlabeled_flags[flag_size].size();
    //cerr << "Type size " << type_size << " for " << g_unlabeled_flags[flag_size][i].print() << endl;
    int mapping[flag_size];
    flag F;
    
    int type_size = type.labeled_vertices_cnt();
    
    if (type_size != 0)
    {
    
        for (int i = 0; i < to_label_size; i++)
        {
            //if (verbose_output)
            //     cerr << "Flag size "<<flag_size << " type size " << type_size << " labeling " << i << "/" << to_label_size << endl;
        
            for (int j = 0; j < flag_size; j++) mapping[j]=j;
        
            do {
                F.as_subflag(g_unlabeled_flags[flag_size][i], mapping, flag_size, type_size); // taking a subflag
                //cerr << "Found " << F.print() << " X " << F.have_same_type(type) << " " << !find_flag_in_list(F, flag_list) << endl;
                if (F.have_same_type(type) && find_flag_in_list(F, flag_list) == -1)
                    flag_list.push_back(F);
            } while ( std::next_permutation(mapping,mapping+flag_size) );
        }
    }
    else
    {
        //cerr << "Bubu " << flag_size << " "  << g_unlabeled_flags[flag_size].size() << endl;
        //flag_list.erase();
        flag_list = g_unlabeled_flags[flag_size];
    }
    
    
    if (dump_flags_to_file(filename,flag_list))
    {
        cerr << "Generated and written flags to " << filename << endl;
        return;
    }    
}



double unlabel_F1(const flag &F1, int remaining_labeled)
{
#ifdef G_ORDERED_VERTICES
    assert(0);
#endif
    
    flag H = F1;
    H.m_Theta = remaining_labeled;
    
    return P_F1_IN_H(F1, H);
}


    
// f is an unlabeled flag
void count_flag_products(stringstream &ss, int matrixID, const flag &f)
{    
    int subset_split[V];
    int mapping_F1[V];
    int mapping_F2[V];
    flag F1, F2; 
 
    // Try to find decomposition of vertices of f into 3 sets.
    // first is of size typesize and the other two are of sizes privatesize
    // Then typesize+privatesize creates one flag.
    
    // We go through all possible type sizes
    for (int typesize = f.m_vertices-2; typesize >= 0; typesize -= 2)
    {
        vector<pair<string,int> > found_pairs;

        int privatesize = (f.m_vertices-typesize)/2; 
        int totalsize = typesize+privatesize;  // size of F1 and F2
        int label_f1 = typesize;
        int label_f2 = typesize+1;

#ifdef G_ORDERED_VERTICES
        for (int i = 0; i < typesize; i++) subset_split[i] = -1; // still same order
#else
        for (int i = 0; i < typesize; i++) subset_split[i] = i;
#endif
        assert(typesize+2*privatesize < V);
        for (int i = 0; i < privatesize; i++)
        {   
            subset_split[typesize+i] = label_f1;
            subset_split[typesize+i+privatesize] = label_f2; // I don't know why the warning is here...
        }
        
        do {
            //std::cout << subset_split[0] << ' ' << subset_split[1] << ' ' << subset_split[2] << ' ' << subset_split[3] << '\n';
        
            // Construction of inverz mapping
#ifdef G_ORDERED_VERTICES
            int inF1 = 0;
            int inF2 = 0;
            int ThetaF1 = 0;
            int ThetaF2 = 0;
            for (int i = 0; i < f.m_vertices; i++)
            {
                if (subset_split[i] == -1)
                {
                    mapping_F1[inF1] = i;
                    mapping_F2[inF2] = i;
                    ThetaF1 |= 1 << inF1++;
                    ThetaF2 |= 1 << inF2++;
                }
                if (subset_split[i] == label_f1) mapping_F1[inF1++] = i; 
                if (subset_split[i] == label_f2) mapping_F2[inF2++] = i; 
            }
            
            F1.as_subflag(f,mapping_F1, totalsize, ThetaF1);
            
            int typeID = get_flag_type_in_list(F1, g_flags);
            if (typeID == -2) continue;
#ifdef G_NOT_ALL_FLAGS_USED
            if (typeID == -1) continue;
#endif            
            assert (typeID != -1);
            
            F2.as_subflag(f,mapping_F2, totalsize, ThetaF2);
            
            if (!F2.have_same_type(F1))
            {
                cerr << typesize << " " << privatesize << endl;
                cerr << f.print() << endl;
                cerr << F1.print() << "   " << F2.print() << endl;
                
                for (int i = 0; i < f.m_vertices;i++)
                    cerr << subset_split[i];
                cerr << endl;
                
                assert(0);
            }
#else
            int inF1 = typesize;
            int inF2 = typesize;            
            for (int i = 0; i < f.m_vertices; i++)
            {
                if (subset_split[i] < typesize) 
                {
                    mapping_F1[subset_split[i]] = i;
                    mapping_F2[subset_split[i]] = i;
                }
                if (subset_split[i] == label_f1) mapping_F1[inF1++] = i; 
                if (subset_split[i] == label_f2) mapping_F2[inF2++] = i; 
            }

            F1.as_subflag(f,mapping_F1, totalsize, typesize);
        
            int typeID = get_flag_type_in_list(F1, g_flags);
            if (typeID == -2) continue;
#ifdef G_NOT_ALL_FLAGS_USED
            if (typeID == -1) continue;
#endif            
            assert (typeID != -1);
            
            F2.as_subflag(f,mapping_F2, totalsize, typesize);
#endif 

            int idF1 = find_flag_in_list(F1,g_flags[typeID]);
            int idF2 = find_flag_in_list(F2,g_flags[typeID]);
            if (idF1 > idF2 ) continue; // count only once when idF1 <= idF2         
            if (idF1 == -1)
            {
#ifdef G_NOT_ALL_FLAGS_USED
                continue;
#endif                
                cerr << "Missing " << F1.print() << endl;
            }
            assert(idF1 >= 0 && idF2 >= 0); 
            
            stringstream what_found_ss;
            what_found_ss << typeID+1 << " " << idF1+1 << " " << idF2+1;
            string what_found = what_found_ss.str();
            
            bool known = false;
            for(int i = 0; i < (int)found_pairs.size(); i++)
            {
                if (found_pairs[i].first == what_found) 
                {
                    found_pairs[i].second++;
                    known = true;
                    break;
                }
            }
            if (known == false)
            {
                found_pairs.push_back(make_pair(what_found,1));
            }
            
        } while ( std::next_permutation(subset_split,subset_split+f.m_vertices) );
        
        for(int i = 0; i < (int)found_pairs.size(); i++)
        {
            ss << matrixID << " " << found_pairs[i].first << " " <<  found_pairs[i].second << endl; 
        }
    }
}


double compute_objective_function_for_SDP(const flag &g)
{
	double total_density = 0;	
#ifdef USE_COUNT_INSTEAD_OF_DENSITY_IN_OBJECTIVE
	for (int i = 0; i < (int)g_objective_combination.size(); i++)
	{
		//total_density += g.count_subflag(g_objective_combination[i].g)*g_objective_combination[i].coefficient;
        total_density += g_objective_combination[i].coefficient * P_F1_IN_H(g_objective_combination[i].g, g);
	}	
	return total_density;	
#endif

	for (int i = 0; i < (int)g_objective_combination.size(); i++)
	{
//		total_density += g.density_subflag(g_objective_combination[i].g)*g_objective_combination[i].coefficient;
        total_density += g_objective_combination[i].coefficient * P_F1_IN_H(g_objective_combination[i].g, g);
	}
	
	return total_density;
}



void generate_all_types_containing_one_subtype(int big_type_size, const flag &type, vector<flag> &flag_list)
{
    flag g;
    g.set_vertices_and_Theta(big_type_size,big_type_size);
    g.copy_from(type);
    
    extensions_of_g(g,flag_list);
}




// According to http://plato.asu.edu/ftp/sdpa_format.txt
//
// Solved program
//   max C.X
//   subject to A_1.X = b_1
//              A_2.X = b_2
//              A_mdim.X = b_mdim
//   where X is positive semidefinite matrix
//
// In our UPPER BOUND application
//
//      min    t
//      s.t.  D_i + [A.X]_i <= t
//
// We add a slack variable s_i to make the <= to = and move D_i (density - constant to the other side)
// and change to maximization:
//
//      max   -t
//      s.t.  [A.X]_i + s_i - t = -D_i
//
//   C.X is just entry 0 2 1 1 -1.0 which means that in the second block,
//   the first variable is with -1. Se we are maximizing -d which corresponds to minimizing d
//
// Now LOWER BOUND application
//
//      max    t
//      s.t.  D_i - [A.X]_i  >= t
//
// We add a slack variable s_i to make the >= to = and move D_i (density - constant to the other side)
// and change to maximization:
//
//      max   t
//      s.t.  [A.X]_i + s_i + t = D_i
//
// Finally, Additional constraints applications:
// Say we want to add  G >= c. Then we could add
//  k(G(sampled in H_i) -c) >= 0
//
// For lower bound we get
//     [A.X]_i + k*(G_i-c) + s_i + t = D_i
//
//
void print_CSDP_specific_part(int Kn, bool upper_bound = true, ostream &ostr = cout, bool verbose_output = true)
{
    int mdim = 0;
    mdim = (int)g_unlabeled_flags[Kn].size();
    
    ostr << mdim << endl; //<<" =mdim" << endl; // number of constraint matrices
    
    stringstream ssblocks; // String stream for all blocks in the matrix
    int blocks = 0; // number of blocks in ssblocks
 
    // First blocks are flags products:
    for (int f = 0; f < (int)g_flags.size(); f++) ssblocks <<  g_flags[f].size() << " ";
    blocks = (int)g_flags.size();
    
    // next we have one diagonal block of slack variables
    ssblocks << -1-mdim << " ";
    blocks++;


    ostr << blocks << endl;
    ostr << ssblocks.str() << endl;
 
    for (unsigned int i = 0; i < g_unlabeled_flags[Kn].size(); i++)
    {
        ostr.precision(17);
        if (upper_bound) ostr << -1*smart_round(compute_objective_function_for_SDP(g_unlabeled_flags[Kn][i])) << " "; // outputing b_i
        else             ostr << smart_round(compute_objective_function_for_SDP(g_unlabeled_flags[Kn][i])) << " "; // outputing b_i
    }
    ostr << endl;
    
    if (upper_bound) ostr << "0 " << (int)g_flags.size()+1 << " 1 1 -1" << endl; // Objective function
    else             ostr << "0 " << (int)g_flags.size()+1 << " 1 1 1" << endl; // Objective function


    // printing of subflags
#pragma omp parallel for ordered schedule(dynamic)
        for (int i = 0; i < (int)g_unlabeled_flags[Kn].size(); i++)
        {            
            //cout << "flag: "; c4free_subrgaphs[e][i].print();
            if (verbose_output) cerr << "Computing specific part " << i+1 << "/" << g_unlabeled_flags[Kn].size() << endl;
            stringstream ss;
            
            int matrixID = i + 1;
         
            if (upper_bound) ss << matrixID << " " << (int)g_flags.size()+1 << " 1 1 -1" << endl;
            else             ss << matrixID << " " << (int)g_flags.size()+1 << " 1 1 1" << endl;
            
            ss << matrixID << " " << (int)g_flags.size()+1 << " " << matrixID+1 << " " <<  matrixID+1 << " 1" << endl;
			
        
#pragma omp ordered
{
            ostr << ss.str();
}
        }
    
}

void print_CSDP_flag_products_part(int Kn, bool upper_bound = true, ostream &ostr = cout, bool verbose_output = true, int lowest_not_computed=0)
{
#pragma omp parallel for ordered schedule(dynamic)
    for (int i = lowest_not_computed; i < (int)g_unlabeled_flags[Kn].size(); i++)
    {            
        if (verbose_output) cerr << "Computing flag products " << i+1 << "/" << g_unlabeled_flags[Kn].size() << endl;

        stringstream ss;
        
        int matrixID = i + 1;
        
#ifdef G_FLAG_PRODUCTS_SLOW_LOW_MEMORY       
         for (int f = 0; f < (int)g_flags.size(); f++)
         {
             for (unsigned int x = 0; x < g_flags[f].size(); x++)
                 for (unsigned int y = x; y < g_flags[f].size(); y++)
                 {
         
                     double PF = P_F1_F2_IN_H(g_flags[f][x], g_flags[f][y], g_unlabeled_flags[Kn][i]);
                     //					if (x == y) PF = PF*0.5;
                     if (PF != 0)
                     {
                         ss << matrixID <<  " " << f+1 << " " << x+1 << " " << y+1 << " " <<  PF << endl;
                     }
                 }
         }
#else
        count_flag_products(ss, matrixID, g_unlabeled_flags[Kn][i]);
#endif        
        		
#pragma omp ordered
        {
            ostr << ss.str();
        }
    }
    
}

void print_CSDP(int Kn, bool upper_bound = true, ostream &ostr = cout, bool verbose_output = true, bool use_sdp_temp=false)
{
    cerr << "Computing specific part of the SDP..." << endl;
    print_CSDP_specific_part(Kn, upper_bound, ostr, verbose_output);
    
    if (!use_sdp_temp)
    {
        cerr << "Computing flag products part of the SDP..." << endl;
        print_CSDP_flag_products_part(Kn, upper_bound, ostr, verbose_output);
        return;
    }
    
    cerr << "Getting flag products for SDP..." << endl;    
    
    stringstream filename;
    
    filename << filename_prefix() << "__n" << Kn << "_sdp_products.txt";

    // The sdp_product_file may not contain all products if the computation
    // was accidentally interrupted. We try to go from we stopped...
    // First find were we stopped
    
    int lowest_not_computed = 0;

    ifstream sdp_product_file;
    sdp_product_file.open (filename.str().c_str(),ifstream::in);
    while (sdp_product_file.good())
    {
        int id, type, f1, f2, d;
        sdp_product_file >> id >> type >> f1 >> f2 >> d;
        if (!sdp_product_file.good()) break; // if the read failed, do not use it...
        if (id > lowest_not_computed)
        {
            lowest_not_computed = id; // note that in file the ID is +1
        }
    }
    sdp_product_file.close();
    
    if (lowest_not_computed < (int)g_unlabeled_flags[Kn].size())
    {
        if (lowest_not_computed != 0)
        {
            cerr << "Warning: file " << filename.str() << " contains only products up to " << lowest_not_computed << " out of " << (int)g_unlabeled_flags[Kn].size() << endl;
            cerr << "          We assume it happened by accident so the rows starting with " << lowest_not_computed << " will be deleted and recomputed" << endl;
            stringstream mv_command;
            mv_command << "rm -f "<< filename.str() << "~ ;  mv " << filename.str() << " " << filename.str() << "~";
            cerr << "          Executing " << mv_command.str() << endl;
            if (system(mv_command.str().c_str()) != 0)
            {
                cerr << " Execution failed" << endl;
                exit(1);
            }
            // We remove the last line since it may be incoplete and not catched by the grep
            stringstream grep_command;
            grep_command << "sed \\$d " << filename.str() << "~ |   grep -v '^" << lowest_not_computed << " ' "  << " >" << filename.str();
            cerr << "          Executing " << grep_command.str() << endl;
            if (system(grep_command.str().c_str()) != 0)
            {
                cerr << " Execution failed" << endl;
                exit(1);
            }
            lowest_not_computed--;
        }
     
        ofstream sdp_product_file_out;
        sdp_product_file_out.open (filename.str().c_str(),ofstream::out|ofstream::app);
        if (!sdp_product_file_out.good())
        {
            cerr << "Failed creating file with sdp products " << filename.str() << endl;
            exit(1);
        }
        
        cerr << "Computing flag products of the SDP to file " << filename.str() << endl;
        print_CSDP_flag_products_part(Kn, upper_bound, sdp_product_file_out, verbose_output, lowest_not_computed);
        sdp_product_file_out.close();
    }

   
    sdp_product_file.open (filename.str().c_str(),ifstream::in);
    if (!sdp_product_file.good())
    {
        cerr << "Failed obtaining file with sdp products " << filename.str() << endl;
        exit(1);
    }
    
    cerr << "Copying flag products of the SDP from " << filename.str() << endl;
    ostr << sdp_product_file.rdbuf();
    sdp_product_file.close();
    
}





// The following function megres dest_flags with src_flags and clears src_flags
void merge_flags_lists_parallel(vector<flag> &dest_flags, vector<flag> &src_flags)
{
    
    //    cout << flag_list.size() << endl;
    
    vector<flag> new_flags;
    new_flags.reserve(src_flags.size());
    
#pragma omp parallel for
    for (int k = 0; k < (int)src_flags.size(); k++)
    {
        if (!g_already_in_known_flags(src_flags[k],dest_flags))
        {
#pragma omp critical (merging_lists)
            {
                new_flags.push_back(src_flags[k]);
            }
        }
    }
//#ifdef DONT_USE_C11    
    dest_flags.insert(dest_flags.end(), new_flags.begin(), new_flags.end() );
    src_flags.clear();
    vector<flag>().swap( src_flags ); // free the memory
//#else
//    dest_flags.insert(dest_flags.end(), std::make_move_iterator(new_flags.begin()), std::make_move_iterator(new_flags.end()) );                                
//    src_flags.clear();  
//    src_flags.shrink_to_fit(); // free the memory - does not work well on icc
//#endif  
}





// Not paralel version - very slow and not completely correct - in particular for ordered version.
void generate_labeled_flags_old(int flag_size, int type_size, bool verbose_output)
{
#ifdef G_ORDERED_VERTICES
    assert(0);
#endif
    int to_label_size = (int)g_unlabeled_flags[flag_size].size();
    //cerr << "Type size " << type_size << " for " << g_unlabeled_flags[flag_size][i].print() << endl;
    int mapping[flag_size];
    flag F;
    
    for (int i = 0; i < to_label_size; i++)
    {
        if (verbose_output)
            cerr << "Flag size "<<flag_size << " type size " << type_size << " labeling " << i << "/" << to_label_size << endl;

        for (int j = 0; j < flag_size; j++) mapping[j]=j;
        
        do {
            F.as_subflag(g_unlabeled_flags[flag_size][i], mapping, flag_size, type_size); // taking a subflag
            include_flag_in_list_if_new(F,g_flags);
        } while ( std::next_permutation(mapping,mapping+flag_size) );
    }
}


//vector<flag> g_types[V];
void create_types_of_size(int type_size)
{
    if (g_types[type_size].size() != 0) return;


    // Look if there are already some flags with given types...
    for (int i = 0; i < (int)g_flags.size(); i++)
    {
        if (g_flags[i].size() == 0) continue;
        
        if (g_flags[i][0].labeled_vertices_cnt() == type_size)
        {
            flag type;
            g_flags[i][0].get_type_subflag(type);
            g_types[type_size].push_back(type);
        }
    }
    if (g_types[type_size].size() != 0) return;
    
    
    // if not, just label graphs of apropriate size
    g_types[type_size] = g_unlabeled_flags[type_size];

    
#pragma omp parallel for
    for (int i = 0; i < (int)g_types[type_size].size(); i++)
    {
        g_types[type_size][i].make_all_vertices_labeled();
        //
        cerr << "New type: " << g_types[type_size][i].print() << endl;
    }
}

int get_reduced_type_id(const flag &f)
{
    int type_size = f.labeled_vertices_cnt();
    
    //cerr << "Testing size " << type_size << endl;
    
    for (int i = 0; i < (int)g_types[type_size].size(); i++)
    {
        if (f.have_same_type(g_types[type_size][i]))
        {
            return i;
        }
    }    
    return -1;
}



// 
void generate_labeled_flags(int flag_size, int type_size, bool verbose_output)
{
    create_types_of_size(type_size);
    
    int to_label_size = (int)g_unlabeled_flags[flag_size].size();
    
    // 
    vector< vector<flag> >  *flags;
    flags = new vector< vector<flag> >[to_label_size];
    
    int types = (int)g_types[type_size].size();
    
    // Avoid copying flags
    for (int i = 0; i < to_label_size; i++)
    {
        flags[i].resize(types);
    }
    
#ifdef G_ORDERED_VERTICES
    int labeled[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    for (int j = 0; j < type_size; j++)
    {
        labeled[flag_size-1-j] = 1;
    }    
    do {
        //for (int j = 0; j < flag_size; j++) cerr << labeled[j] << " ";
        //cerr << endl;
        for (int i = 0; i < to_label_size; i++)
        {
            flag F;
            F = g_unlabeled_flags[flag_size][i];
            int Theta=0;
            for (int j = 0; j < flag_size; j++)
            {
                Theta = (Theta << 1) | labeled[j];
            }
            F.set_Theta(Theta);

            int type_of_F = get_reduced_type_id(F);
            if (type_of_F == -1)
            {
                cerr << "Testing labeled flag " << F.print() << " type " << type_of_F << endl;
                flag g;
                F.get_type_subflag(g);
                cerr << "Type: " << g.print() << endl;
            }
            if (type_of_F == -1) continue;
            //if (!g_already_in_known_flags(F,flags[i][type_of_F])) // they are always new
            {
                //cerr << "Adding " << F.print() << endl;
                //flags[i][type_of_F].push_back(F);
                include_flag_in_list_if_new(F, g_flags);
            }
        }
    } while ( std::next_permutation(labeled,labeled+flag_size) );
    
#else
    cerr << "Generating labeled flags for Cauchy-Swartz in parallel " << endl;
#pragma omp parallel for ordered
    for (int i = 0; i < to_label_size; i++)
    {
        if (verbose_output)
            cerr << "Flag size "<<flag_size << " type size " << type_size << " labeling " << i << "/" << to_label_size << endl;

        //cerr << "Type size " << type_size << " for " << g_unlabeled_flags[flag_size][i].print() << endl;
        // Permutations for labeling... Could be improved to distinguish only the first type_size vertices.
        int mapping[flag_size];
        flag F;
        for (int j = 0; j < flag_size; j++) mapping[j]=j;
        
        //int found_good_type = 0;
        do {
            F.as_subflag(g_unlabeled_flags[flag_size][i], mapping, flag_size, type_size); // taking a subflag
            int type_of_F = get_reduced_type_id(F);
            if (type_of_F == -1)
            {
                // We have each type only once -  3 3   1 1  2  and  3 3   1 2  1 are different types but enough to use just one
                continue;
            }
            //found_good_type++;
            if (!g_already_in_known_flags(F,flags[i][type_of_F]))
            {
                    flags[i][type_of_F].push_back(F);
            }
            //include_flag_in_list_if_new(F,flags[i],false);
        } while ( std::next_permutation(mapping,mapping+flag_size) );
        //cout << "Found good type " << found_good_type << endl;
    }
    
    // Todo: this might be also maybe paralelized
    for (int i = 0; i < to_label_size; i++)
    {
        for (int j = 0; j < (int)flags[i].size(); j++)
            for (int k = 0; k < (int)flags[i][j].size(); k++)
                include_flag_in_list_if_new(flags[i][j][k], g_flags);
        
        //merge_typed_flags_lists_parallel(g_flags, flags[i]);
    }
#endif
  
    delete[] flags;

    //cerr << "done " << endl;
}


//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP
//////////////////////////////////////////////////////// LOAD & DUMP


bool load_objective_from_file(const string &filename, bool verbose_output)
{
    ifstream infile;
    infile.open (filename.c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file with objective function " << filename << endl;
        return false;
    }
    
    cerr << "Loading objective function from file " << filename << endl;
        
    flag_and_coefficient fc;
    
    infile >> fc.coefficient;
    assert (fc.coefficient != 0);
    while (infile && fc.g.load_from_stream(infile,-1,-1))
    {
        // check if the flag is labeled
        if (fc.g.labeled_vertices_cnt() != 0)
        {
            cerr << "WARNING: You are using labeled flags in th objective function" << endl;
        }
        // check if there are no repetitions in the objective function..
        for (int i = 0; i < (int)g_objective_combination.size(); i++)
        {
            if (g_objective_combination[i].g.is_isomorphic_to(fc.g))
            {
                cerr << "WARNING: In the objecive function, flags " << g_objective_combination[i].g.print() << " and " <<
                fc.g.print() << " are the same " << endl;
                //exit(1);
            }
        }
        g_objective_combination.push_back(fc);
        infile >> fc.coefficient;
        
        if (fc.coefficient == 0)
        {
            //load_linear_constraints_from_stream(infile, verbose_output);
            break;
        }
    }
    
    infile.close();
    
    cerr << "# of entries in objective is " << g_objective_combination.size() << endl;
    
    return true;
}



// final_flags means that the flags will not be used any further
// for genertin other flags. Important when combined with G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY
bool load_unlabeled_flags_from_file(int sizeKn, bool final_flags = true)
{
    stringstream filename;
    
    filename << filename_prefix() << "__n" << sizeKn << "_unlabeled.txt";
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file with unlabeled flags " << filename.str() << endl;
        return false;
    }
    
    cerr << "Loading unlabeled flags from file " << filename.str() << endl;
    
    // For large instances, it really helps to reserve the space in containers
    //  before loading... so we read the file twice
    int sizes_cnt[V];
    for (int i = 0; i < V; i++) sizes_cnt[i] = 0;
    flag g;
    while (g.load_from_stream(infile,-1,0))
    {
        sizes_cnt[g.m_vertices]++;
    }
    for (int i = 0; i < V; i++) g_unlabeled_flags[i].reserve(sizes_cnt[i]);
    
//    infile.seek(0);
    infile.close();
    infile.open (filename.str().c_str(), ifstream::in);
    
    
    while (g.load_from_stream(infile,-1,0))
    {
        
        assert(sizeKn >= g.m_vertices);
        
#ifdef G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY
        if (!final_flags) // Drop graphs that contain edges from blow-up
        {
#ifdef G_COLORED_EDGES
            if (g.m_colored_edges[1] > 0) continue;
#endif
#ifdef G_COLORED_3EDGES
            if (g.m_colored_3edges[1] > 0) continue;
#endif
        }
#endif
#ifdef G_CHECK_FOR_FORBIDDEN_WHILE_LOADING_FLAGS
        if (is_flag_forbidden(g))
        {
            continue;
        }
#endif      
#ifdef G_REMOVE_DUPLICATES_WHILE_LOADING_FLAGS        
        bool new_flag = true;
        for (int i = 0; i < (int)g_unlabeled_flags[g.m_vertices].size();i++)
        {
            if (g.is_isomorphic_to(g_unlabeled_flags[g.m_vertices][i]))
            {
                new_flag = false;
            }
        }
        if (!new_flag) continue;
#endif   
        g_unlabeled_flags[g.m_vertices].push_back(g);
    }
    
    infile.close();
    
    for (int i = 0; i <= sizeKn; i++)
    {
        cerr << "Loaded # of unlabeled flags of size " << i << " is " << g_unlabeled_flags[i].size() << endl;
    }
    
    return true;
}



bool load_labeled_flags_from_file(int sizeKn)
{
    stringstream filename;
    filename << filename_prefix() << "__n" << sizeKn << "_labeled.txt";
    
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file with labeled flags " << filename.str() << endl;
        return false;
    }

   cerr << "Loading labeled flags from file " << filename.str() << endl;

    
   flag f;
	while (f.load_from_stream(infile,-1,-1))
	{
		include_flag_in_list(f, g_flags);
	}
	
	infile.close();
    
	return true;
}

bool dump_labeled_flags(int sizeKn)
{
    stringstream filename;
    filename << filename_prefix() << "__n" << sizeKn << "_labeled.txt";
    
    ofstream outfile;
    outfile.open (filename.str().c_str(), ofstream::out);
    if (!outfile.good())
    {
        cerr << "Failed opening file " << filename.str() << endl;
        return false;
    }
    
    cerr << "Writing labeled flags to file " << filename.str() << endl;
    
    for (int f = 0; f < (int)g_flags.size(); f++)
    {
        for (unsigned int x = 0; x < g_flags[f].size(); x++)
        {
            outfile << g_flags[f][x].print() << endl;
        }
        if (f < (int)g_flags.size()-1) outfile << endl;
    }
    
    outfile.close();
    
    return true;
}


bool dump_unlabeled_flags(int sizeKn)
{
    stringstream filename;
    filename << filename_prefix() << "__n" << sizeKn << "_unlabeled.txt";
    
    ofstream outfile;
    outfile.open (filename.str().c_str(), ofstream::out);
    if (!outfile.good())
    {
        cerr << "Failed opening file " << filename.str() << endl;
        return false;
    }
    
    cerr << "Writing unlabeled flags to file " << filename.str() << endl;
    
    for (int f = 0; f < V; f++)
    {
        for (unsigned int x = 0; x < g_unlabeled_flags[f].size(); x++)
        {
            outfile << g_unlabeled_flags[f][x].print() << endl;
        }
    }
    
    outfile.close();
    
    return true;
}


bool compare_flag_sizes(const flag&f1, const flag &f2)
{
    return f1.m_vertices < f2.m_vertices;
}

void sort_flags_by_size(vector<flag> &flag_list)
{
    sort(flag_list.begin(), flag_list.end(), compare_flag_sizes);
}


#ifdef G_USE_PERMITTED_SUBFLAGS
void load_permitted()
{
    stringstream filename;
    filename <<  filename_prefix() << "__permitted.txt";
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file " << filename.str() << " no permitted structures are used" << endl;
        return;
    }
    
    int flags_in_file = 0;
    
    flag h;
    while (h.load_from_stream(infile,-1,0))
    {
        flags_in_file++;
        // check if h is not already there
        if (!(g_already_in_known_flags(h,g_permitted_subflags[h.m_vertices])))
        {
            g_permitted_subflags[h.m_vertices].push_back(h);
            //cerr << "Loaded forbidden graph " << h.print() << endl;
        }
    }
    
    infile.close();
    
    for (int i = 0; i < V; i++)
    {
        if (g_permitted_subflags[i].size() != 0)
        {
            cerr << "Loaded " << g_permitted_subflags[i].size() << " permitted subflags of size " <<  i  << " graphs from " << filename.str() << endl;
        }
    }
}
#endif

void load_forbidden()
{
    stringstream filename;
    filename <<  filename_prefix() << "__forbidden.txt";
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file " << filename.str() << " no forbidden structures are used" << endl;
        return;
    }
    
    int flags_in_file = 0;
    
    flag h;
    while (h.load_from_stream(infile,-1,0))
    {
        flags_in_file++;
        // check if h is not already there
        if (!(g_already_in_known_flags(h,g_forbidden_subflags)))
        {
            g_forbidden_subflags.push_back(h);
            g_forbidden_subflags_by_size[h.m_vertices].push_back(h);
            //cerr << "Loaded forbidden graph " << h.print() << endl;
        }
    }
    
    infile.close();
    
    // Idea is that it is faster to test smaller subflags than bigger
    // so lets first test smaller ones
    sort_flags_by_size(g_forbidden_subflags);
    
    cerr << "Loaded " << g_forbidden_subflags.size() << " forbidden graphs from " << filename.str() << endl;
    if (flags_in_file != (int)g_forbidden_subflags.size())
    {
        cerr << "File " << filename.str() << " contains " << flags_in_file-g_forbidden_subflags.size() << " duplicate  graphs." << endl;    
    }
    
    //for (int i = 0; i < g_forbidden_subflags.size(); i++)
    //{
    //    cerr << g_forbidden_subflags[i].print() << endl;
    //}
}

#ifdef G_COLORED_EDGES_BLIND
void load_blind_colorededge_permutations()
{
    stringstream filename;
    filename <<  filename_prefix() << "__edgeblind_permutations.txt";
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file " << filename.str() << " for allowed edge color permutations (dont forget identity!)" << endl;
        cerr << "Using ALL possible permutations...." << endl;
        int color_permutation[] = {0,1,2,3,4,5,7,8,9,10,11,12,13,14}; // longer than needed
        
        do {
            vector<int> color_perm;
            for (int i = 0; i < COLORS_EDGES; i++)
            {
                color_perm.push_back(color_permutation[i]);
            }
            g_allowed_edgecolor_permutations.push_back(color_perm);
        } while ( std::next_permutation(color_permutation+1,color_permutation+COLORS_EDGES));

        return;
    }
    
    while (infile.good())
    {
        vector<int> color_perm;
        for (int i = 0; i < COLORS_EDGES; i++)
        {
            int color;
            infile >> color;
            assert (color >= 0 && color < COLORS_EDGES);
            color_perm.push_back(color);
        }
        if (!infile.good()) break;
        g_allowed_edgecolor_permutations.push_back(color_perm);
    }
    
    infile.close();
    
    cerr << "# of edge colored permutations: " << g_allowed_edgecolor_permutations.size() << endl;
}
#endif


#ifdef G_COLORED_VERTICES_BLIND
void load_blind_coloredvertex_permutations()
{
    stringstream filename;
    filename <<  filename_prefix() << "__vertexblind_permutations.txt";
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file " << filename.str() << " for allowed vertex color permutations (don't forget identity!)" << endl;
        cerr << "Assuming that all permutations are fine" << endl;
        int color_permutation[] = {0,1,2,3,4,5,7,8,9,10}; // longer than needed
        
        do {
            vector<int> color_perm;
            for (int i = 0; i < COLORS_VERTICES; i++)
            {
                color_perm.push_back(color_permutation[i]);
            }
            g_allowed_vertexcolor_permutations.push_back(color_perm);
        } while ( std::next_permutation(color_permutation+1,color_permutation+COLORS_VERTICES));
        return;
    }
    
    while (infile.good())
    {
        vector<int> color_perm;
        for (int i = 0; i < COLORS_VERTICES; i++)
        {
            int color;
            infile >> color;
            assert (color >= 0 && color < COLORS_VERTICES);
            color_perm.push_back(color);
        }
        if (!infile.good()) break;
        g_allowed_vertexcolor_permutations.push_back(color_perm);
    }
    
    infile.close();
    
    cerr << "# of vertex colored permutations: " << g_allowed_vertexcolor_permutations.size() << endl;
}
#endif


#ifdef G_COLORED_3EDGES_BLIND
void load_blind_colored3edge_permutations()
{
    stringstream filename;
    filename <<  filename_prefix() << "__3edgeblind_permutations.txt";
    
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file " << filename.str() << " for allowed edge color permutations (dont forget identity!)" << endl;
        exit(1);
    }
    
    while (infile.good())
    {
        vector<int> color_perm;
        for (int i = 0; i < COLORS_3EDGES; i++)
        {
            int color;
            infile >> color;
            assert (color >= 0 && color < COLORS_3EDGES);
            color_perm.push_back(color);
        }
        if (!infile.good()) break;
        g_allowed_3edgecolor_permutations.push_back(color_perm);
    }
    
    infile.close();
    
    cerr << "# of 3edge colored permutations: " << g_allowed_3edgecolor_permutations.size() << endl;
}
#endif


//////////////////////////////////////////////////////// 





void try_extensions_of_g_to_last_vertex_edges(flag &g, vector<flag> &flag_list)
{
#ifdef G_3EDGES  
#ifdef G_COLORED_EDGES
#error I dont know how to generate 2edges and 3edges at the same time.
#endif
#endif    

    int v = g.m_vertices;    
#ifdef G_3EDGES       
    try_add_3edges(g,v-1,0,1,flag_list);
    return;
#endif
#ifdef G_COLORED_3EDGES
    try_color_3edges(g,v-1,0,1,flag_list);
    return;
#endif    
#ifdef G_4EDGES       
    try_add_4edges(g,v-1,0,1,2,flag_list);
    return;
#endif
#ifdef G_COLORED_EDGES
    try_color_edge(g,v-1,0,flag_list);
    return;
#endif
    
    assert(0);
}


void try_extensions_of_g_to_last_vertex(flag &g, vector<flag> &flag_list)
{   
#ifdef G_COLORED_VERTICES        
    int v = g.m_vertices;
    for (int c = 1; c < COLORS_VERTICES; c++)
    {
        g.color_vertex(v-1,c);
        try_extensions_of_g_to_last_vertex_edges(g, flag_list);
    }
#else
    try_extensions_of_g_to_last_vertex_edges(g, flag_list);
#endif
}


void dump_vector_of_flags(vector <flag> *flag_lists, int previous_size, int sizeKn)
{
    stringstream filename;
    filename << filename_prefix() << "__n" << sizeKn << "_unlabeled_dump.txt";
    
    ofstream outfile;
    outfile.open (filename.str().c_str(), ofstream::out);
    if (!outfile.good())
    {
        cerr << "Failed opening file " << filename.str() << endl;
        return;
    }
    
    // dumping of flag lists...
    for (int i = 0; i < previous_size; i++)
    {
        for (int f = 0; f < (int)flag_lists[i].size(); f++)
        {
            outfile << flag_lists[i][f].print() << endl;
        }
        outfile << endl;
    }    
    
    outfile.close();
}

void generate_unlabeled_flags_of_size(int i, bool verbose_output, bool dump_unlabeled_while_generating)
{
    int previous_size = (int)g_unlabeled_flags[i-1].size();
    
    //vector <flag> flag_lists[previous_size]; // does not work on mac (llvm :-( )
    vector <flag> *flag_lists;
    flag_lists = new vector<flag>[previous_size];
    if (flag_lists == NULL)
    {
        cerr << "Memory allocation failed" << endl;
        exit(1);
    }
    
    //            #pragma omp parallel for
#pragma omp parallel for ordered schedule(dynamic)
    for (int j = 0; j < (int)g_unlabeled_flags[i-1].size(); j++)
    {
        flag_lists[j].reserve(previous_size);
        
        flag g;
        g.set_vertices(i);
        g.copy_from(g_unlabeled_flags[i-1][j]);
        //vector <flag> valid_extensions_of_g;
        //cout << "going for " << j << endl;
        try_extensions_of_g_to_last_vertex(g, flag_lists[j]);
        if (verbose_output)
        {
            cerr << "generated size " << i << ": " << j  << "/" << (int)g_unlabeled_flags[i-1].size() << " found " << flag_lists[j].size() << " flags " << endl;
        }
    }
    if (verbose_output)
    {
        cerr << "Flags generated. Merging part starts." << endl;
    }
    
    if (dump_unlabeled_while_generating)
        dump_vector_of_flags(flag_lists, previous_size, i);
    
    for (int power = 1; power < previous_size; power *= 2)
    {
        if (verbose_output)
        {
            cerr << "Trying " << power << " in " << previous_size << endl;
        }                
        for (int j = 0; j < previous_size; j += 2*power)
        {   
            if (j+power < previous_size)
            {
                if (verbose_output)
                {
                    cerr << "Merging  " << j << "<-" << j+power << "  of sizes " <<  flag_lists[j].size() << " " << flag_lists[j+power].size() << endl;
                }   
                merge_flags_lists_parallel(flag_lists[j],flag_lists[j+power]);
                //flag_lists[j+power].clear();
            }
        }
        if (dump_unlabeled_while_generating)
            dump_vector_of_flags(flag_lists, previous_size, i);
    }
    
    g_unlabeled_flags[i].swap(flag_lists[0]);
    
    delete[] flag_lists;
}



void extensions_of_g_with_print(flag &g, bool extension_count_copies)
{
    vector<flag> flag_list;
    
    extensions_of_g(g, flag_list);
    
    for (unsigned int x = 0; x < flag_list.size(); x++)
    {
        if (extension_count_copies)
        {
            cout << flag_list[x].count_labeled_copies_of(g) << "    ";
        }
        cout << flag_list[x].print() << endl;
    }
}


#ifndef G_DISABLE_MAIN

int main(int argc, char *argv[])
{
    // Parse command line
    int Kn = 0;
    string objective_file = "";
    string output_file = "";
    bool verbose_output = false;
    bool use_sdp_temp = false;
    bool extension = false;
    bool extension_count_copies = false;
    flag g_extensions;
    bool forbidden_test = false;
    flag g_forbidden_test;    
    bool use_sdpa = false;
    
    bool dump_unlabeled = false;
    bool dump_unlabeled_while_generating = false;
    bool quit_after_generating_unlabeled = false;
    bool force_generate_flags = false;
    
    bool upper_bound = false;
    bool lower_bound = false;
    bool force_generating_constriants = false;
    
    bool don_run_csdp = false;
    
    string csdp_binary="csdp";
    
    
    
    if (argc <= 1)
    {
        cerr << endl;
        cerr << "Usage: " << argv[0] << " -n SIZE_OF_UNLABELED_FLAGS [-obj OBJECTIVE_FILENAME] [-v] [-o OUTPUT_DAT-S] [-lb | -ub]" << endl;
        cerr << " -n SIZE_OF_UNLABELED_FLAGS   Size of unlabeled graphs" << endl;
        cerr << " -obj OBJECTIVE_FILENAME      File containing objective function" << endl;
        cerr << " -v                           Display progress when creating SDP and generating graphs" << endl;
        cerr << " -o OUTPUT_DAT-S              Write sdp to a file OUTPUT_DAT-S, usable 'stdout', default test.dat-s" << endl;
        cerr << " -ub                          Creates SDP computing UPPER BOUND on the objective" << endl;        
        cerr << " -lb                          Creates SDP computing LOWER BOUND on the objective" << endl;
        cerr << " -fp                          Stores flag products or tries to reuse them - usefull for repeated computations, eats more space" << endl;
        cerr << " -e GRAPH                     Prints all extnesions of GRAPH and exits" << endl;        
        cerr << " -ec GRAPH                    Prints all extnesions of GRAPH and # of labeled copies of GRAPH in extensions and exits" << endl;        
        cerr << " -ft GRAPH                    Forbiden Test - checks if GRAPH is forbidden" << endl;        
        cerr << " -fgf                         Force generating flags - not loading them from files" << endl;
        cerr << " -fgc                         Force generating constraints - not loading from file" << endl;
        cerr << " -du                          Dump unlabeled even when loaded from file - usefull for example if the input contain also forbidden graphs, then this can work a 1 time filter" << endl;
        cerr << " -dug                         Dump unlabeled flags as they are being generated - usefull for example if you want to enumerate all graphs and not really compute the rest" << endl;
        cerr << " -qagu                        Quit after generating unlabeled flags" << endl;
        cerr << " -ncsdp                       Do not run csdp at the end" << endl;
        cerr << " -csdp CSDPBIN                Command to run when running csdp. Default is csdp." << endl;
        cerr << " -sdpa                        use sdpa rather than csdp" << endl;
        cerr << endl;
        cerr << "Filenames:" << endl;
        cerr << "    " << filename_prefix() << "__?n_unlabeled.txt    unlabeled flags" << endl; 
        cerr << "    " << filename_prefix() << "__?n_labeled.txt      labeled flags" << endl;
        cerr << "    " << filename_prefix() << "__?n_sdp_products.txt part of sdp" << endl;
        cerr << "    " << filename_prefix() << "__forbidden.txt       forbidden flags" << endl;
        cerr << "    " << filename_prefix() << "__objective.txt       default objective" << endl;
#ifdef G_USE_PERMITTED_SUBFLAGS
        cerr << "    " << filename_prefix() << "__permitted.txt       permitted subflags" << endl;
#endif        
#ifdef G_COLORED_EDGES_BLIND
        cerr << "    " << filename_prefix() << "__edgeblind_permutations.txt  allowed perms" << endl;
#endif        
#ifdef G_COLORED_3EDGES_BLIND
        cerr << "    " << filename_prefix() << "__3edgeblind_permutations.txt  allowed perms" << endl;
#endif
        cerr << endl;
        return 0;
    }  
    
    
    int i = 1;
    while (i < argc)
    {        
        if (string(argv[i]) == "-n") {
            Kn = atoi(argv[i+1]);
            i++;
        } else if (string(argv[i]) == "-obj") {
            objective_file = argv[i + 1];
            i++;            
        } else if (string(argv[i]) == "-o") {
            output_file = argv[i + 1];
            i++;
        } else if (string(argv[i]) == "-csdp") {
            csdp_binary = argv[i + 1];
            i++;
        } else if (string(argv[i]) == "-v") {
            verbose_output = true;
        } else if (string(argv[i]) == "-lb") {
            lower_bound = true;
        } else if (string(argv[i]) == "-ub") {
            upper_bound = true;
        } else if (string(argv[i]) == "-du") {
            dump_unlabeled = true;
        } else if (string(argv[i]) == "-dug") {
            dump_unlabeled_while_generating = true;
        } else if (string(argv[i]) == "-qagu") {
            quit_after_generating_unlabeled = true;
        } else if (string(argv[i]) == "-ncsdp") {
            don_run_csdp = true;            
        } else if (string(argv[i]) == "-sdpa") {
            use_sdpa = true;     
        } else if (string(argv[i]) == "-fgc") {
            force_generating_constriants = true;
        } else if (string(argv[i]) == "-fp") {
            use_sdp_temp = true;
        } else if (string(argv[i]) == "-extensions" || string(argv[i]) == "-e" || string(argv[i]) == "-ec") {
            extension_count_copies = string(argv[i]) == "-ec";
            g_extensions.load_from_string(argv[i + 1]);
            extension = true;
            i++;
        } else if (string(argv[i]) == "-forbidden_test" || string(argv[i]) == "-ft" ) {
            g_forbidden_test.load_from_string(argv[i + 1]);
            forbidden_test = true;
            i++;
        } else {
            cerr << "Unsupported argument " << argv[i] << " " << endl;
            return 0;
        }
        i++;
    }

    
    
#ifdef G_COLORED_EDGES_BLIND    
    load_blind_colorededge_permutations();
#endif
#ifdef G_COLORED_3EDGES_BLIND    
    load_blind_colored3edge_permutations();
#endif
#ifdef G_COLORED_VERTICES_BLIND    
    load_blind_coloredvertex_permutations();
#endif

    
    load_forbidden();

#ifdef G_USE_PERMITTED_SUBFLAGS
    load_permitted();    
#endif  
    
    
    if (extension)
    {
        extensions_of_g_with_print(g_extensions,extension_count_copies);
        return 0;
    }

    if (forbidden_test)
    {
        if (is_flag_forbidden(g_forbidden_test))
        {
            cerr << "-ft graph forbidden " << g_forbidden_test.print() << endl;
        } else {
            cerr << "-ft graph not forbidden " << g_forbidden_test.print() << endl;    
        }
        return 0;
    }
    
    
    
    if (objective_file == "")
    {
        stringstream filename;
        filename <<  filename_prefix() << "__objective.txt";
    
        objective_file = filename.str();
        cerr << "Objective file not provided, using default filename " << objective_file << endl;
    }
    
    if (!load_objective_from_file(objective_file, verbose_output))
    {
        return 1;
    }    
    
    assert(Kn <= V);
        

    
    // Getting unlabeled flags...
    if (force_generate_flags || !load_unlabeled_flags_from_file(Kn))
    {
#ifdef G_CROSSINGS
        cerr << "Cannot generate unlabeled flags. Must be provided." << endl;
        return 1;
#endif        
        // Try to reuse previous graphs...
        int loaded_size = 0;
    
        // blow-upping and generating would not work correctly...
        if (!force_generate_flags)
        {
            loaded_size = Kn-1;
            cerr << "Trying to load file with smaller flags..." << endl;
            while (loaded_size > 0 && !load_unlabeled_flags_from_file(loaded_size,false)) loaded_size--;
        }
        
        if (loaded_size == 0)
        {
            cerr << "Unable to load any smaller graphs. Starting from empty graph." << endl;
            flag g_zero;
            g_zero.set_vertices_and_Theta(0,0);
            g_unlabeled_flags[0].push_back(g_zero);
        }
            
        cerr << "Generating unlabeled flags of sizes " << loaded_size+1 << " to " << Kn << endl;
        
        for (int i = loaded_size+1; i <= Kn; i++)
        {
            generate_unlabeled_flags_of_size(i, verbose_output, dump_unlabeled_while_generating);
            cerr << "Unlabbeled flags of size "<< i << ": " << g_unlabeled_flags[i].size() << endl;
        } 
            
#ifdef G_USE_FIRST_EDGE_COLOR_FOR_BLOWUP_ONLY
        // Create blow-ups            
        for (int i = 1; i <= Kn; i++)
        {
            cerr << "Before blow-up unlabbeled flags of size "<< i << ": " << g_unlabeled_flags[i].size() << endl;
            generate_blowups_by_one_vertex(g_unlabeled_flags[i-1], g_unlabeled_flags[i]);
            cerr << "After blow-up unlabbeled flags of size "<< i << ": " << g_unlabeled_flags[i].size() << endl;
        }
#endif
        
        dump_unlabeled_flags(Kn);
    } 
    else
    {
        if (dump_unlabeled) dump_unlabeled_flags(Kn);
    }
    if (quit_after_generating_unlabeled)
    {
        cerr << "Quitting after generating unlabeled flags." << endl;
        return 0;
    }
    
    // labeled flags
    g_flags.reserve(MAX_FLAG_TYPES); // Should make live much faster
    if (force_generate_flags || !load_labeled_flags_from_file(Kn))
    {
        cerr << "Generating labeled flags..." << endl;
        // Generating all labeled flags:
        int last_types = (int)g_flags.size();
//// TODO: Make this parallel        
        for (int i = 1; i <= Kn/2; i++)
        {
            cerr << "Labeled flags " << Kn-i << ":" << Kn-2*i << " give ... ";
            generate_labeled_flags(Kn-i,Kn-2*i, verbose_output);
            int gain = (int)g_flags.size() - last_types;
            cerr << gain << " types ";
            for (int j = last_types; j < (int)g_flags.size(); j++)
                cerr << g_flags[j].size() << " ";
            cerr << endl;
            last_types = (int)g_flags.size();
        }
        dump_labeled_flags(Kn);
    }
    cerr << "Labeled flag have " << (int)g_flags.size() << " types. Counts: ";
    for (int i = 0; i < (int)g_flags.size(); i++) cerr << " " << g_flags[i].size();
    cerr << endl;
   
    
    if (!upper_bound && !lower_bound)
    {
        cerr << "I dont know if I should try upper bound our lower bound - you need to tell me using -lb or -ub" << endl;
        return 1;
    }
    
    if (upper_bound && lower_bound)
    {
        cerr << "I'm sorry, but I cannot do both upper and lower bound at the same time." << endl;
        return 1;
    }    
    
    
    // create result_file
    if (output_file == "" )
    {
        stringstream filename;
        filename << "SDP_n" << Kn << "_";
        if (lower_bound) filename << "LB_";
        else  filename << "UB_";
        filename <<  objective_file << ".dat-s"; 
        output_file = filename.str();
    }
    stringstream filename;
    filename << output_file << ".result";
    string result_file = filename.str();   
    
    
    if (output_file == "stdout")
    {
        cerr << "Generating SDP program to stdout..." << endl;
        print_CSDP(Kn,upper_bound,cout,verbose_output,use_sdp_temp);

#ifdef USE_COUNT_INSTEAD_OF_DENSITY_IN_OBJECTIVE
        cerr << "Warning - using count instead of density in SDP" << endl;
#endif
        
    } else {
        if (output_file == "" )
        {
            stringstream filename;
            filename << "SDP_n" << Kn << "_";
            if (lower_bound) filename << "LB_";
            else  filename << "UB_";
            filename <<  objective_file << ".dat-s"; 
            output_file = filename.str();
        }
        stringstream filename;
        filename << output_file << ".result";
        string result_file = filename.str();
               
        ofstream outfile;
        outfile.open (output_file.c_str(), ofstream::out);
        if (!outfile.good())
        {
            cerr << "Failed opening file " << output_file << endl;
            return -1;
        }
        cerr << "Generating SDP program to " << output_file << endl;
        print_CSDP(Kn,upper_bound,outfile,verbose_output,use_sdp_temp);
        outfile.close();
        
#ifdef USE_COUNT_INSTEAD_OF_DENSITY_IN_OBJECTIVE
        cerr << "Warning - using count instead of density in SDP" << endl;
#endif
        
        if (!don_run_csdp)
        {
            if (use_sdpa)
            {
                execlp("sdpa","sdpa","-ds",output_file.c_str(),"-o",result_file.c_str(),(char *)NULL);                
            }
            else
            {
                execlp(csdp_binary.c_str(),"csdp",output_file.c_str(),result_file.c_str(),(char *)NULL);
            }
        }
    }
    
    cerr << "Done." << endl;
    
    
    return 0;    
}

#else
#error "Bubu"
#endif


